searchData={"content_type":"text/markdown","items":[{"doc":"A HTTP toolkit for building API clients using middlewares.","ref":"Tesla.html","title":"Tesla","type":"module"},{"doc":"Use `Tesla.client/2` to build a client with the given middleware and adapter.","ref":"Tesla.html#module-building-api-client","title":"Building API client - Tesla","type":"module"},{"doc":"```elixir\ndefmodule ExampleApi do\n  def client do\n    Tesla.client([\n      {Tesla.Middleware.BaseUrl, \"http://api.example.com\"},\n      Tesla.Middleware.JSON\n    ])\n  end\n\n  def fetch_data(client) do\n    Tesla.get(client, \"/data\")\n  end\nend\n```\n\nNow you can use `ExampleApi.client/0` to make requests to the API.\n\n```elixir\nclient = ExampleApi.client()\nExampleApi.fetch_data(client)\n```","ref":"Tesla.html#module-examples","title":"Examples - Tesla","type":"module"},{"doc":"It is also possible to do request directly with `Tesla` module.\n\n```elixir\nTesla.get(\"https://example.com\")\n```","ref":"Tesla.html#module-direct-usage","title":"Direct usage - Tesla","type":"module"},{"doc":"By default `Tesla` is using `Tesla.Adapter.Httpc`, because `:httpc` is\nincluded in Erlang/OTP and does not require installation of any additional\ndependency. It can be changed globally with config:\n\n```elixir\nconfig :tesla, :adapter, Tesla.Adapter.Mint\n```","ref":"Tesla.html#module-default-adapter","title":"Default adapter - Tesla","type":"module"},{"doc":"","ref":"Tesla.html#build_adapter/1","title":"Tesla.build_adapter/1","type":"function"},{"doc":"","ref":"Tesla.html#build_client/2","title":"Tesla.build_client/2","type":"function"},{"doc":"Builds a URL from the given `t:Tesla.Env.t/0` struct.\n\nCombines the `url` and `query` fields, and allows specifying the `encoding`\nstrategy before calling `build_url/3`.","ref":"Tesla.html#build_url/1","title":"Tesla.build_url/1","type":"function"},{"doc":"Builds URL with the given URL and query params.\n\nUseful when you need to create a URL with dynamic query params from a Keyword\nlist\n\nAllows to specify the `encoding` strategy to be one either `:www_form` or\n`:rfc3986`. Read more about encoding at `URI.encode_query/2`.\n\n- `url` - the base URL to which the query params will be appended.\n- `query` - a list of key-value pairs to be encoded as query params.\n- `encoding` - the encoding strategy to use. Defaults to `:www_form`","ref":"Tesla.html#build_url/3","title":"Tesla.build_url/3","type":"function"},{"doc":"iex> Tesla.build_url(\"https://api.example.com\", [user: 3, page: 2])\n    \"https://api.example.com?user=3&page=2\"\n\nURL that already contains query params:\n\n    iex> url = \"https://api.example.com?user=3\"\n    iex> Tesla.build_url(url, [page: 2, status: true])\n    \"https://api.example.com?user=3&page=2&status=true\"\n\nDefault encoding `:www_form`:\n\n    iex> Tesla.build_url(\"https://api.example.com\", [user_name: \"John Smith\"])\n    \"https://api.example.com?user_name=John+Smith\"\n\nSpecified encoding strategy `:rfc3986`:\n\n    iex> Tesla.build_url(\"https://api.example.com\", [user_name: \"John Smith\"], :rfc3986)\n    \"https://api.example.com?user_name=John%20Smith\"","ref":"Tesla.html#build_url/3-examples","title":"Examples - Tesla.build_url/3","type":"function"},{"doc":"Dynamically build client from list of middlewares and/or adapter.\n\n```\n# add dynamic middleware\nclient = Tesla.client([{Tesla.Middleware.Headers, [{\"authorization\", token}]}])\nTesla.get(client, \"/path\")\n\n# configure adapter in runtime\nclient = Tesla.client([], Tesla.Adapter.Hackney)\nclient = Tesla.client([], {Tesla.Adapter.Hackney, pool: :my_pool})\nTesla.get(client, \"/path\")\n\n# complete module example\ndefmodule MyApi do\n  @middleware [\n    {Tesla.Middleware.BaseUrl, \"https://example.com\"},\n    Tesla.Middleware.JSON,\n    Tesla.Middleware.Logger\n  ]\n\n  @adapter Tesla.Adapter.Hackney\n\n  def new(opts) do\n    # do any middleware manipulation you need\n    middleware = [\n      {Tesla.Middleware.BasicAuth, username: opts[:username], password: opts[:password]}\n    ] ++ @middleware\n\n    # allow configuring adapter in runtime\n    adapter = opts[:adapter] || @adapter\n\n    # use Tesla.client/2 to put it all together\n    Tesla.client(middleware, adapter)\n  end\n\n  def get_something(client, id) do\n    # pass client directly to Tesla.get/2\n    Tesla.get(client, \"/something/#{id}\")\n    # ...\n  end\nend\n\nclient = MyApi.new(username: \"admin\", password: \"secret\")\nMyApi.get_something(client, 42)\n```","ref":"Tesla.html#client/2","title":"Tesla.client/2","type":"function"},{"doc":"Perform a DELETE request.\n\nSee `request/1` or `request/2` for options definition.\n\n    delete(\"/users\")\n    delete(\"/users\", query: [scope: \"admin\"])\n    delete(client, \"/users\")\n    delete(client, \"/users\", query: [scope: \"admin\"])\n    delete(client, \"/users\", body: %{name: \"Jon\"})","ref":"Tesla.html#delete/3","title":"Tesla.delete/3","type":"function"},{"doc":"Perform a DELETE request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    delete!(\"/users\")\n    delete!(\"/users\", query: [scope: \"admin\"])\n    delete!(client, \"/users\")\n    delete!(client, \"/users\", query: [scope: \"admin\"])\n    delete!(client, \"/users\", body: %{name: \"Jon\"})","ref":"Tesla.html#delete!/3","title":"Tesla.delete!/3","type":"function"},{"doc":"","ref":"Tesla.html#delete_header/2","title":"Tesla.delete_header/2","type":"function"},{"doc":"","ref":"Tesla.html#encode_query/2","title":"Tesla.encode_query/2","type":"function"},{"doc":"","ref":"Tesla.html#t:encoding_strategy/0","title":"Tesla.encoding_strategy/0","type":"type"},{"doc":"Perform a GET request.\n\nSee `request/1` or `request/2` for options definition.\n\n    get(\"/users\")\n    get(\"/users\", query: [scope: \"admin\"])\n    get(client, \"/users\")\n    get(client, \"/users\", query: [scope: \"admin\"])\n    get(client, \"/users\", body: %{name: \"Jon\"})","ref":"Tesla.html#get/3","title":"Tesla.get/3","type":"function"},{"doc":"Perform a GET request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    get!(\"/users\")\n    get!(\"/users\", query: [scope: \"admin\"])\n    get!(client, \"/users\")\n    get!(client, \"/users\", query: [scope: \"admin\"])\n    get!(client, \"/users\", body: %{name: \"Jon\"})","ref":"Tesla.html#get!/3","title":"Tesla.get!/3","type":"function"},{"doc":"Returns value of header specified by `key` from `:headers` field in `Tesla.Env`.","ref":"Tesla.html#get_header/2","title":"Tesla.get_header/2","type":"function"},{"doc":"# non existing header\n    iex> env = %Tesla.Env{headers: [{\"server\", \"Cowboy\"}]}\n    iex> Tesla.get_header(env, \"some-key\")\n    nil\n\n    # existing header\n    iex> env = %Tesla.Env{headers: [{\"server\", \"Cowboy\"}]}\n    iex> Tesla.get_header(env, \"server\")\n    \"Cowboy\"\n\n    # first of multiple headers with the same name\n    iex> env = %Tesla.Env{headers: [{\"cookie\", \"chocolate\"}, {\"cookie\", \"biscuits\"}]}\n    iex> Tesla.get_header(env, \"cookie\")\n    \"chocolate\"","ref":"Tesla.html#get_header/2-examples","title":"Examples - Tesla.get_header/2","type":"function"},{"doc":"","ref":"Tesla.html#get_headers/2","title":"Tesla.get_headers/2","type":"function"},{"doc":"Perform a HEAD request.\n\nSee `request/1` or `request/2` for options definition.\n\n    head(\"/users\")\n    head(\"/users\", query: [scope: \"admin\"])\n    head(client, \"/users\")\n    head(client, \"/users\", query: [scope: \"admin\"])\n    head(client, \"/users\", body: %{name: \"Jon\"})","ref":"Tesla.html#head/3","title":"Tesla.head/3","type":"function"},{"doc":"Perform a HEAD request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    head!(\"/users\")\n    head!(\"/users\", query: [scope: \"admin\"])\n    head!(client, \"/users\")\n    head!(client, \"/users\", query: [scope: \"admin\"])\n    head!(client, \"/users\", body: %{name: \"Jon\"})","ref":"Tesla.html#head!/3","title":"Tesla.head!/3","type":"function"},{"doc":"Options that may be passed to a request function. See `request/2` for detailed descriptions.","ref":"Tesla.html#t:option/0","title":"Tesla.option/0","type":"type"},{"doc":"Perform a OPTIONS request.\n\nSee `request/1` or `request/2` for options definition.\n\n    options(\"/users\")\n    options(\"/users\", query: [scope: \"admin\"])\n    options(client, \"/users\")\n    options(client, \"/users\", query: [scope: \"admin\"])\n    options(client, \"/users\", body: %{name: \"Jon\"})","ref":"Tesla.html#options/3","title":"Tesla.options/3","type":"function"},{"doc":"Perform a OPTIONS request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    options!(\"/users\")\n    options!(\"/users\", query: [scope: \"admin\"])\n    options!(client, \"/users\")\n    options!(client, \"/users\", query: [scope: \"admin\"])\n    options!(client, \"/users\", body: %{name: \"Jon\"})","ref":"Tesla.html#options!/3","title":"Tesla.options!/3","type":"function"},{"doc":"Perform a PATCH request.\n\nSee `request/1` or `request/2` for options definition.\n\n    patch(\"/users\", %{name: \"Jon\"})\n    patch(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    patch(client, \"/users\", %{name: \"Jon\"})\n    patch(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"Tesla.html#patch/4","title":"Tesla.patch/4","type":"function"},{"doc":"Perform a PATCH request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    patch!(\"/users\", %{name: \"Jon\"})\n    patch!(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    patch!(client, \"/users\", %{name: \"Jon\"})\n    patch!(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"Tesla.html#patch!/4","title":"Tesla.patch!/4","type":"function"},{"doc":"Perform a POST request.\n\nSee `request/1` or `request/2` for options definition.\n\n    post(\"/users\", %{name: \"Jon\"})\n    post(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    post(client, \"/users\", %{name: \"Jon\"})\n    post(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"Tesla.html#post/4","title":"Tesla.post/4","type":"function"},{"doc":"Perform a POST request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    post!(\"/users\", %{name: \"Jon\"})\n    post!(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    post!(client, \"/users\", %{name: \"Jon\"})\n    post!(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"Tesla.html#post!/4","title":"Tesla.post!/4","type":"function"},{"doc":"Perform a PUT request.\n\nSee `request/1` or `request/2` for options definition.\n\n    put(\"/users\", %{name: \"Jon\"})\n    put(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    put(client, \"/users\", %{name: \"Jon\"})\n    put(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"Tesla.html#put/4","title":"Tesla.put/4","type":"function"},{"doc":"Perform a PUT request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    put!(\"/users\", %{name: \"Jon\"})\n    put!(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    put!(client, \"/users\", %{name: \"Jon\"})\n    put!(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"Tesla.html#put!/4","title":"Tesla.put!/4","type":"function"},{"doc":"","ref":"Tesla.html#put_body/2","title":"Tesla.put_body/2","type":"function"},{"doc":"","ref":"Tesla.html#put_header/3","title":"Tesla.put_header/3","type":"function"},{"doc":"","ref":"Tesla.html#put_headers/2","title":"Tesla.put_headers/2","type":"function"},{"doc":"Adds given key/value pair to `:opts` field in `Tesla.Env`.\n\nUseful when there's a need to store additional middleware data in `Tesla.Env`","ref":"Tesla.html#put_opt/3","title":"Tesla.put_opt/3","type":"function"},{"doc":"iex> %Tesla.Env{opts: []} |> Tesla.put_opt(:option, \"value\")\n    %Tesla.Env{opts: [option: \"value\"]}","ref":"Tesla.html#put_opt/3-examples","title":"Examples - Tesla.put_opt/3","type":"function"},{"doc":"Perform a request.","ref":"Tesla.html#request/2","title":"Tesla.request/2","type":"function"},{"doc":"- `:method` - the request method, one of [`:head`, `:get`, `:delete`, `:trace`, `:options`, `:post`, `:put`, `:patch`]\n- `:url` - either full url e.g. \"http://example.com/some/path\" or just \"/some/path\" if using `Tesla.Middleware.BaseUrl`\n- `:query` - a keyword list of query params, e.g. `[page: 1, per_page: 100]`\n- `:headers` - a keyword list of headers, e.g. `[{\"content-type\", \"text/plain\"}]`\n- `:body` - depends on used middleware:\n    - by default it can be a binary\n    - if using e.g. JSON encoding middleware it can be a nested map\n    - if adapter supports it it can be a Stream with any of the above\n- `:opts` - custom, per-request middleware or adapter options","ref":"Tesla.html#request/2-options","title":"Options - Tesla.request/2","type":"function"},{"doc":"ExampleApi.request(method: :get, url: \"/users/path\")\n\n    # use shortcut methods\n    ExampleApi.get(\"/users/1\")\n    ExampleApi.post(client, \"/users\", %{name: \"Jon\"})","ref":"Tesla.html#request/2-examples","title":"Examples - Tesla.request/2","type":"function"},{"doc":"Perform request and raise in case of error.\n\nThis is similar to `request/2` behaviour from Tesla 0.x\n\nSee `request/2` for list of available options.","ref":"Tesla.html#request!/2","title":"Tesla.request!/2","type":"function"},{"doc":"","ref":"Tesla.html#run/2","title":"Tesla.run/2","type":"function"},{"doc":"","ref":"Tesla.html#run_default_adapter/2","title":"Tesla.run_default_adapter/2","type":"function"},{"doc":"Perform a TRACE request.\n\nSee `request/1` or `request/2` for options definition.\n\n    trace(\"/users\")\n    trace(\"/users\", query: [scope: \"admin\"])\n    trace(client, \"/users\")\n    trace(client, \"/users\", query: [scope: \"admin\"])\n    trace(client, \"/users\", body: %{name: \"Jon\"})","ref":"Tesla.html#trace/3","title":"Tesla.trace/3","type":"function"},{"doc":"Perform a TRACE request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    trace!(\"/users\")\n    trace!(\"/users\", query: [scope: \"admin\"])\n    trace!(client, \"/users\")\n    trace!(client, \"/users\", query: [scope: \"admin\"])\n    trace!(client, \"/users\", body: %{name: \"Jon\"})","ref":"Tesla.html#trace!/3","title":"Tesla.trace!/3","type":"function"},{"doc":"","ref":"Tesla.Client.html","title":"Tesla.Client","type":"module"},{"doc":"","ref":"Tesla.Client.html#t:adapter/0","title":"Tesla.Client.adapter/0","type":"type"},{"doc":"Returns the client's adapter in the same form it was provided.\nThis can be used to copy an adapter from one client to another.","ref":"Tesla.Client.html#adapter/1","title":"Tesla.Client.adapter/1","type":"function"},{"doc":"iex> client = Tesla.client([], {Tesla.Adapter.Hackney, [recv_timeout: 30_000]})\n    iex> Tesla.Client.adapter(client)\n    {Tesla.Adapter.Hackney, [recv_timeout: 30_000]}","ref":"Tesla.Client.html#adapter/1-examples","title":"Examples - Tesla.Client.adapter/1","type":"function"},{"doc":"","ref":"Tesla.Client.html#t:middleware/0","title":"Tesla.Client.middleware/0","type":"type"},{"doc":"Returns the client's middleware in the same form it was provided.\nThis can be used to copy middleware from one client to another.","ref":"Tesla.Client.html#middleware/1","title":"Tesla.Client.middleware/1","type":"function"},{"doc":"iex> middleware = [Tesla.Middleware.JSON, {Tesla.Middleware.BaseUrl, \"https://api.github.com\"}]\n    iex> client = Tesla.client(middleware)\n    iex> Tesla.Client.middleware(client)\n    [Tesla.Middleware.JSON, {Tesla.Middleware.BaseUrl, \"https://api.github.com\"}]","ref":"Tesla.Client.html#middleware/1-examples","title":"Examples - Tesla.Client.middleware/1","type":"function"},{"doc":"","ref":"Tesla.Client.html#t:t/0","title":"Tesla.Client.t/0","type":"type"},{"doc":"This module defines a `t:Tesla.Env.t/0` struct that stores all data related to request/response.","ref":"Tesla.Env.html","title":"Tesla.Env","type":"module"},{"doc":"- `:method` - method of request. Example: `:get`\n- `:url` - request url. Example: `\"https://www.google.com\"`\n- `:query` - list of query params.\n  Example: `[{\"param\", \"value\"}]` will be translated to `?params=value`.\n  Note: query params passed in url (e.g. `\"/get?param=value\"`) are not parsed to `query` field.\n- `:headers` - list of request/response headers.\n  Example: `[{\"content-type\", \"application/json\"}]`.\n  Note: request headers are overridden by response headers when adapter is called.\n- `:body` - request/response body.\n  Note: request body is overridden by response body when adapter is called.\n- `:status` - response status. Example: `200`\n- `:opts` - list of options. Example: `[adapter: [recv_timeout: 30_000]]`","ref":"Tesla.Env.html#module-fields","title":"Fields - Tesla.Env","type":"module"},{"doc":"","ref":"Tesla.Env.html#t:body/0","title":"Tesla.Env.body/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:client/0","title":"Tesla.Env.client/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:headers/0","title":"Tesla.Env.headers/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:method/0","title":"Tesla.Env.method/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:opts/0","title":"Tesla.Env.opts/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:param/0","title":"Tesla.Env.param/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:query/0","title":"Tesla.Env.query/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:result/0","title":"Tesla.Env.result/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:runtime/0","title":"Tesla.Env.runtime/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:stack/0","title":"Tesla.Env.stack/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:status/0","title":"Tesla.Env.status/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:t/0","title":"Tesla.Env.t/0","type":"type"},{"doc":"","ref":"Tesla.Env.html#t:url/0","title":"Tesla.Env.url/0","type":"type"},{"doc":"Mock adapter for better testing.","ref":"Tesla.Mock.html","title":"Tesla.Mock","type":"module"},{"doc":"```elixir\n# config/test.exs\nconfig :tesla, adapter: Tesla.Mock\n\n# in case MyClient defines specific adapter with `adapter SpecificAdapter`\nconfig :tesla, MyClient, adapter: Tesla.Mock\n```","ref":"Tesla.Mock.html#module-setup","title":"Setup - Tesla.Mock","type":"module"},{"doc":"```elixir\ndefmodule MyAppTest do\n  use ExUnit.Case\n\n  setup do\n    Tesla.Mock.mock(fn\n      %{method: :get} ->\n        %Tesla.Env{status: 200, body: \"hello\"}\n    end)\n\n    :ok\n  end\n\n  test \"list things\" do\n    assert {:ok, env} = MyApp.get(\"...\")\n    assert env.status == 200\n    assert env.body == \"hello\"\n  end\nend\n```","ref":"Tesla.Mock.html#module-examples","title":"Examples - Tesla.Mock","type":"module"},{"doc":"```elixir\n# Match on method & url and return whole Tesla.Env\nTesla.Mock.mock(fn\n  %{method: :get, url: \"http://example.com/list\"} ->\n    %Tesla.Env{status: 200, body: \"hello\"}\nend)\n\n# You can use any logic required\nTesla.Mock.mock(fn env ->\n  case env.url do\n    \"http://example.com/list\" ->\n      %Tesla.Env{status: 200, body: \"ok!\"}\n\n    _ ->\n      %Tesla.Env{status: 404, body: \"NotFound\"}\n  end\nend)\n\n\n# mock will also accept short version of response\n# in the form of {status, headers, body}\nTesla.Mock.mock(fn\n  %{method: :post} -> {201, %{}, %{id: 42}}\nend)\n\n# mock will also accept error tuples in the form\n# of {:error, reason}\nTesla.Mock.mock(fn\n  %{method: :post} -> {:error, :timeout}\nend)\n```","ref":"Tesla.Mock.html#module-setting-up-mocks","title":"Setting up mocks - Tesla.Mock","type":"module"},{"doc":"By default, mocks are bound to the current process,\ni.e. the process running a single test case.\nThis design allows proper isolation between test cases\nand make testing in parallel (`async: true`) possible.\n\nWhile this style is recommended, there is one drawback:\nif Tesla client is called from different process\nit will not use the setup mock.\n\nTo solve this issue it is possible to setup a global mock\nusing `mock_global/1` function.\n\n```elixir\ndefmodule MyTest do\n  use ExUnit.Case, async: false # must be false!\n\n  setup_all do\n    Tesla.Mock.mock_global fn\n      env -> # ...\n    end\n\n    :ok\n  end\n\n  # ...\nend\n```\n\n**WARNING**: Using global mocks may affect tests with local mock\n(because of fallback to global mock in case local one is not found)","ref":"Tesla.Mock.html#module-global-mocks","title":"Global mocks - Tesla.Mock","type":"module"},{"doc":"","ref":"Tesla.Mock.html#call/2","title":"Tesla.Mock.call/2","type":"function"},{"doc":"Return JSON response.\n\nExample\n\n    import Tesla.Mock\n\n    mock fn\n      %{url: \"/ok\"} -> json(%{\"some\" => \"data\"})\n      %{url: \"/404\"} -> json(%{\"some\" => \"data\"}, status: 404)\n    end","ref":"Tesla.Mock.html#json/2","title":"Tesla.Mock.json/2","type":"function"},{"doc":"Setup mocks for current test.\n\nThis mock will only be available to the current process.","ref":"Tesla.Mock.html#mock/1","title":"Tesla.Mock.mock/1","type":"function"},{"doc":"Setup global mocks.\n\n**WARNING**: This mock will be available to ALL processes.\nIt might cause conflicts when running tests in parallel!","ref":"Tesla.Mock.html#mock_global/1","title":"Tesla.Mock.mock_global/1","type":"function"},{"doc":"","ref":"Tesla.Mock.html#t:response_opt/0","title":"Tesla.Mock.response_opt/0","type":"type"},{"doc":"","ref":"Tesla.Mock.html#t:response_opts/0","title":"Tesla.Mock.response_opts/0","type":"type"},{"doc":"Return text response.\n\nExample\n\n    import Tesla.Mock\n\n    mock fn\n      %{url: \"/ok\"} -> text(\"200 ok\")\n      %{url: \"/404\"} -> text(\"404 not found\", status: 404)\n    end","ref":"Tesla.Mock.html#text/2","title":"Tesla.Mock.text/2","type":"function"},{"doc":"Multipart functionality.","ref":"Tesla.Multipart.html","title":"Tesla.Multipart","type":"module"},{"doc":"```\nmp =\n  Multipart.new()\n  |> Multipart.add_content_type_param(\"charset=utf-8\")\n  |> Multipart.add_field(\"field1\", \"foo\")\n  |> Multipart.add_field(\"field2\", \"bar\",\n    headers: [{\"content-id\", \"1\"}, {\"content-type\", \"text/plain\"}]\n  )\n  |> Multipart.add_file(\"test/tesla/multipart_test_file.sh\")\n  |> Multipart.add_file(\"test/tesla/multipart_test_file.sh\", name: \"foobar\")\n  |> Multipart.add_file_content(\"sample file content\", \"sample.txt\")\n\nresponse = client.post(url, mp)\n```","ref":"Tesla.Multipart.html#module-examples","title":"Examples - Tesla.Multipart","type":"module"},{"doc":"Add a parameter to the multipart content-type.","ref":"Tesla.Multipart.html#add_content_type_param/2","title":"Tesla.Multipart.add_content_type_param/2","type":"function"},{"doc":"Add a field part.","ref":"Tesla.Multipart.html#add_field/4","title":"Tesla.Multipart.add_field/4","type":"function"},{"doc":"Add a file part. The file will be streamed.","ref":"Tesla.Multipart.html#add_file/3","title":"Tesla.Multipart.add_file/3","type":"function"},{"doc":"- `:name` - name of form param\n- `:filename` - filename (defaults to path basename)\n- `:headers` - additional headers\n- `:detect_content_type` - auto-detect file content-type (defaults to false)","ref":"Tesla.Multipart.html#add_file/3-options","title":"Options - Tesla.Multipart.add_file/3","type":"function"},{"doc":"Add a file part with value.\n\nSame as `add_file/3` but the file content is read from `data` input argument.","ref":"Tesla.Multipart.html#add_file_content/4","title":"Tesla.Multipart.add_file_content/4","type":"function"},{"doc":"- `:name` - name of form param\n- `:headers` - additional headers","ref":"Tesla.Multipart.html#add_file_content/4-options","title":"Options - Tesla.Multipart.add_file_content/4","type":"function"},{"doc":"Create a new Multipart struct to be used for a request body.","ref":"Tesla.Multipart.html#new/0","title":"Tesla.Multipart.new/0","type":"function"},{"doc":"","ref":"Tesla.Multipart.html#t:part_stream/0","title":"Tesla.Multipart.part_stream/0","type":"type"},{"doc":"","ref":"Tesla.Multipart.html#t:part_value/0","title":"Tesla.Multipart.part_value/0","type":"type"},{"doc":"","ref":"Tesla.Multipart.html#t:t/0","title":"Tesla.Multipart.t/0","type":"type"},{"doc":"","ref":"Tesla.Multipart.Part.html","title":"Tesla.Multipart.Part","type":"module"},{"doc":"","ref":"Tesla.Multipart.Part.html#t:t/0","title":"Tesla.Multipart.Part.t/0","type":"type"},{"doc":"Provides utilities for testing Tesla-based HTTP clients.","ref":"Tesla.Test.html","title":"Tesla.Test","type":"module"},{"doc":"Asserts that the current process's mailbox contains a `TeslaMox` message.\nIt uses `assert_received/1` under the hood.","ref":"Tesla.Test.html#assert_received_tesla_call/3","title":"Tesla.Test.assert_received_tesla_call/3","type":"macro"},{"doc":"- `expected_env` - The expected `t:Tesla.Env.t/0` passed to the adapter.\n- `expected_opts` - The expected `t:Tesla.Adapter.options/0` passed to the\n  adapter.\n- `opts` - Extra configuration options.\n  - `:adapter` - Optional. The adapter to expect the call on. Falls back to\n    the `:tesla` application configuration.","ref":"Tesla.Test.html#assert_received_tesla_call/3-parameters","title":"Parameters - Tesla.Test.assert_received_tesla_call/3","type":"macro"},{"doc":"Asserting that the adapter received a `t:Tesla.Env.t/0` struct with a `200`\nstatus:\n\n    defmodule MyTest do\n      use ExUnit.Case, async: true\n\n      require Tesla.Test\n\n      test \"returns a 200 status\" do\n        # given - preconditions\n        Tesla.Test.expect_tesla_call(\n          times: 2,\n          returns: %Tesla.Env{status: 200, body: \"OK\"}\n        )\n\n        # when - run unit of work\n        # ... do some work ...\n        Tesla.post!(\"https://acme.com/users\")\n        # ...\n\n        # then - assertions\n        Tesla.Test.assert_received_tesla_call(expected_env, expected_opts)\n        Tesla.Test.assert_tesla_env(expected_env, %Tesla.Env{\n          url: \"https://acme.com/users\",\n          status: 200,\n          body: \"OK\"\n        })\n        assert expected_opts == []\n        Tesla.Test.assert_tesla_empty_mailbox()\n      end\n    end","ref":"Tesla.Test.html#assert_received_tesla_call/3-examples","title":"Examples - Tesla.Test.assert_received_tesla_call/3","type":"macro"},{"doc":"Asserts that the current process's mailbox does not contain any `Tesla.Test`\nmessages.\n\nThis function is designed to be used in conjunction with\n`Tesla.Test.assert_received_tesla_call/1` for comprehensive request\ntesting.","ref":"Tesla.Test.html#assert_tesla_empty_mailbox/0","title":"Tesla.Test.assert_tesla_empty_mailbox/0","type":"macro"},{"doc":"Asserts that two `t:Tesla.Env.t/0` structs match.","ref":"Tesla.Test.html#assert_tesla_env/3","title":"Tesla.Test.assert_tesla_env/3","type":"function"},{"doc":"- `given_env` - The actual `t:Tesla.Env.t/0` struct received from the request.\n- `expected_env` - The expected `t:Tesla.Env.t/0` struct to compare against.\n- `opts` - Additional options for fine-tuning the assertion (optional).\n  - `:exclude_headers` - A list of header keys to exclude from the assertion.\n\nFor the `body`, the function attempts to parse JSON and URL-encoded content\nwhen appropriate.\n\nThis function is designed to be used in conjunction with\n`Tesla.Test.assert_received_tesla_call/1` for comprehensive request\ntesting.","ref":"Tesla.Test.html#assert_tesla_env/3-parameters","title":"Parameters - Tesla.Test.assert_tesla_env/3","type":"function"},{"doc":"defmodule MyTest do\n      use ExUnit.Case, async: true\n\n      require Tesla.Test\n\n      test \"returns a 200 status\" do\n        given_env = %Tesla.Env{\n          method: :post,\n          url: \"https://acme.com/users\",\n        }\n\n        Tesla.Test.assert_tesla_env(given_env, %Tesla.Env{\n          method: :post,\n          url: \"https://acme.com/users\",\n        })\n      end\n    end","ref":"Tesla.Test.html#assert_tesla_env/3-examples","title":"Examples - Tesla.Test.assert_tesla_env/3","type":"function"},{"doc":"Expects a call on the given adapter using `Mox.expect/4`. Only available when\n`Mox` is loaded.","ref":"Tesla.Test.html#expect_tesla_call/1","title":"Tesla.Test.expect_tesla_call/1","type":"function"},{"doc":"- `:times` - Required. The number of times to expect the call.\n- `:returns` - Required. The value to return from the adapter.\n- `:send_to` - Optional. The process to send the message to. Defaults to\n  the current process.\n- `:adapter` - Optional. The adapter to expect the call on. Falls back to\n  the `:tesla` application configuration.","ref":"Tesla.Test.html#expect_tesla_call/1-options","title":"Options - Tesla.Test.expect_tesla_call/1","type":"function"},{"doc":"Returning a `t:Tesla.Env.t/0` struct with a `200` status:\n\n    Tesla.Test.expect_tesla_call(\n      times: 2,\n      returns: %Tesla.Env{status: 200}\n    )\n\nChanging the `Mox` mocked adapter:\n\n    Tesla.Test.expect_tesla_call(\n      times: 2,\n      returns: %Tesla.Env{status: 200},\n      adapter: MyApp.MockAdapter\n    )","ref":"Tesla.Test.html#expect_tesla_call/1-examples","title":"Examples - Tesla.Test.expect_tesla_call/1","type":"function"},{"doc":"Puts an HTML response.\n\n    iex> Tesla.Test.html(%Tesla.Env{}, \"  Hello, world!  \")\n    %Tesla.Env{\n      body: \"  Hello, world!  \",\n      headers: [{\"content-type\", \"text/html; charset=utf-8\"}],\n      ...\n    }","ref":"Tesla.Test.html#html/2","title":"Tesla.Test.html/2","type":"function"},{"doc":"Puts a JSON response.\n\n    iex> Tesla.Test.json(%Tesla.Env{}, %{\"some\" => \"data\"})\n    %Tesla.Env{\n      body: ~s({\"some\":\"data\"}),\n      headers: [{\"content-type\", \"application/json; charset=utf-8\"}],\n      ...\n    }\n\nIf the body is binary, it will be returned as is and it will not try to encode\nit to JSON.","ref":"Tesla.Test.html#json/2","title":"Tesla.Test.json/2","type":"function"},{"doc":"Puts a text response.\n\n    iex> Tesla.Test.text(%Tesla.Env{}, \"Hello, world!\")\n    %Tesla.Env{\n      body: \"Hello, world!\",\n      headers: [{\"content-type\", \"text/plain; charset=utf-8\"}],\n      ...\n    }","ref":"Tesla.Test.html#text/2","title":"Tesla.Test.text/2","type":"function"},{"doc":"The adapter specification.\n\nAdapter is a module that denormalize request data stored in `Tesla.Env` in order to make\nrequest with lower level http client (e.g. `:httpc` or `:hackney`) and normalize response data\nin order to store it back to `Tesla.Env`. It has to implement `c:Tesla.Adapter.call/2`.","ref":"Tesla.Adapter.html","title":"Tesla.Adapter","type":"behaviour"},{"doc":"Create a module implementing `c:Tesla.Adapter.call/2`.\n\nSee `c:Tesla.Adapter.call/2` for details.","ref":"Tesla.Adapter.html#module-writing-custom-adapter","title":"Writing custom adapter - Tesla.Adapter","type":"behaviour"},{"doc":"defmodule MyProject.CustomAdapter do\n      alias Tesla.Multipart\n\n      @behaviour Tesla.Adapter\n\n      @override_defaults [follow_redirect: false]\n\n      @impl Tesla.Adapter\n      def call(env, opts) do\n        opts = Tesla.Adapter.opts(@override_defaults, env, opts)\n\n        with {:ok, {status, headers, body}} <- request(env.method, env.body, env.headers, opts) do\n          {:ok, normalize_response(env, status, headers, body)}\n        end\n      end\n\n      defp request(_method, %Stream{}, _headers, _opts) do\n        {:error, \"stream not supported by adapter\"}\n      end\n\n      defp request(_method, %Multipart{}, _headers, _opts) do\n        {:error, \"multipart not supported by adapter\"}\n      end\n\n      defp request(method, body, headers, opts) do\n        :lower_level_http.request(method, body, denormalize_headers(headers), opts)\n      end\n\n      defp denormalize_headers(headers), do: ...\n      defp normalize_response(env, status, headers, body), do: %Tesla.Env{env | ...}\n    end","ref":"Tesla.Adapter.html#module-examples","title":"Examples - Tesla.Adapter","type":"behaviour"},{"doc":"Invoked when a request runs.","ref":"Tesla.Adapter.html#c:call/2","title":"Tesla.Adapter.call/2","type":"callback"},{"doc":"- `env` - `t:Tesla.Env.t/0` struct that stores request/response data.\n- `options` - middleware options provided by user.","ref":"Tesla.Adapter.html#c:call/2-arguments","title":"Arguments - Tesla.Adapter.call/2","type":"callback"},{"doc":"Unstructured data passed to the adapter using `opts[:adapter]`.","ref":"Tesla.Adapter.html#t:options/0","title":"Tesla.Adapter.options/0","type":"type"},{"doc":"Helper function that merges all adapter options.","ref":"Tesla.Adapter.html#opts/3","title":"Tesla.Adapter.opts/3","type":"function"},{"doc":"- `defaults` (optional) - useful to override lower level http client default\n  configuration.\n- `env` - `t:Tesla.Env.t()`\n- `opts` - options provided to the adapter from `Tesla.client/2`.","ref":"Tesla.Adapter.html#opts/3-arguments","title":"Arguments - Tesla.Adapter.opts/3","type":"function"},{"doc":"The options are merged in the following order of precedence (highest to lowest):\n\n1. Options from `env.opts[:adapter]` (highest precedence).\n2. Options provided to the adapter from `Tesla.client/2`.\n3. Default options (lowest precedence).\n\nThis means that options specified in `env.opts[:adapter]` will override any\nconflicting options from the other sources, allowing for fine-grained control\non a per-request basis.","ref":"Tesla.Adapter.html#opts/3-precedence-rules","title":"Precedence rules - Tesla.Adapter.opts/3","type":"function"},{"doc":"The middleware specification.\n\nMiddleware is an extension of basic `Tesla` functionality. It is a module that must\nimplement `c:Tesla.Middleware.call/3`.","ref":"Tesla.Middleware.html","title":"Tesla.Middleware","type":"behaviour"},{"doc":"Options can be passed to middleware inside tuple in case of dynamic middleware\n(`Tesla.client/1`):\n\n    Tesla.client([{Tesla.Middleware.BaseUrl, \"https://example.com\"}])","ref":"Tesla.Middleware.html#module-middleware-options","title":"Middleware options - Tesla.Middleware","type":"behaviour"},{"doc":"The order in which middleware is defined matters. Note that the order when _sending_ the request\nmatches the order the middleware was defined in, but the order when _receiving_ the response\nis reversed.\n\nFor example, `Tesla.Middleware.DecompressResponse` must come _after_ `Tesla.Middleware.JSON`,\notherwise the response isn't decompressed before it reaches the JSON parser.","ref":"Tesla.Middleware.html#module-ordering","title":"Ordering - Tesla.Middleware","type":"behaviour"},{"doc":"Writing custom middleware is as simple as creating a module implementing `c:Tesla.Middleware.call/3`.\n\nSee `c:Tesla.Middleware.call/3` for details.","ref":"Tesla.Middleware.html#module-writing-custom-middleware","title":"Writing custom middleware - Tesla.Middleware","type":"behaviour"},{"doc":"defmodule MyProject.InspectHeadersMiddleware do\n      @behaviour Tesla.Middleware\n\n      @impl true\n      def call(env, next, _options) do\n        IO.inspect(env.headers)\n\n        with {:ok, env} <- Tesla.run(env, next) do\n          IO.inspect(env.headers)\n          {:ok, env}\n        end\n      end\n    end","ref":"Tesla.Middleware.html#module-examples","title":"Examples - Tesla.Middleware","type":"behaviour"},{"doc":"Invoked when a request runs.\n\n- (optionally) read and/or writes request data\n- calls `Tesla.run/2`\n- (optionally) read and/or writes response data","ref":"Tesla.Middleware.html#c:call/3","title":"Tesla.Middleware.call/3","type":"callback"},{"doc":"- `env` - `Tesla.Env` struct that stores request/response data\n- `next` - middlewares that should be called after current one\n- `options` - middleware options provided by user","ref":"Tesla.Middleware.html#c:call/3-arguments","title":"Arguments - Tesla.Middleware.call/3","type":"callback"},{"doc":"Adapter for [finch](https://github.com/sneako/finch).\n\nRemember to add `{:finch, \"~> 0.14.0\"}` to dependencies. Also, you need to\nrecompile tesla after adding the `:finch` dependency:\n\n```shell\nmix deps.clean tesla\nmix compile\n```","ref":"Tesla.Adapter.Finch.html","title":"Tesla.Adapter.Finch","type":"module"},{"doc":"In order to use Finch, you must start it and provide a `:name`. For example,\nin your supervision tree:\n\n```elixir\nchildren = [\n  {Finch, name: MyFinch}\n]\n```\n\nYou must provide the same name to this adapter:\n\n```elixir\n# set globally in config/config.exs\nconfig :tesla, :adapter, {Tesla.Adapter.Finch, name: MyFinch}\n\n# set per module\ndefmodule MyClient do\n  def client do\n    Tesla.client([], {Tesla.Adapter.Finch, name: MyFinch})\n  end\nend\n```","ref":"Tesla.Adapter.Finch.html#module-examples","title":"Examples - Tesla.Adapter.Finch","type":"module"},{"doc":"* `:name` - The `:name` provided to Finch (**required**).\n\n## [Finch options](https://hexdocs.pm/finch/Finch.html#request/3)\n\n  * `:pool_timeout` - This timeout is applied when a connection is checked\n    out from the pool. Default value is `5_000`.\n\n  * `:receive_timeout` - The maximum time to wait for a response before\n    returning an error. Default value is `15_000`.","ref":"Tesla.Adapter.Finch.html#module-adapter-specific-options","title":"Adapter specific options - Tesla.Adapter.Finch","type":"module"},{"doc":"Adapter for [gun](https://github.com/ninenines/gun).\n\nRemember to add `{:gun, \"~> 1.3\"}`, `{:idna, \"~> 6.0\"}` and `{:castore, \"~> 0.1\"}` to dependencies.\n\nIn version 1.3 gun sends `host` header with port. Fixed in master branch.\nAlso, you need to recompile tesla after adding `:gun` dependency:\n\n```shell\nmix deps.clean tesla\nmix deps.compile tesla\n```","ref":"Tesla.Adapter.Gun.html","title":"Tesla.Adapter.Gun","type":"module"},{"doc":"```elixir\n# set globally in config/config.exs\nconfig :tesla, :adapter, Tesla.Adapter.Gun\n\n# set per module\ndefmodule MyClient do\n  def client do\n    Tesla.client([], Tesla.Adapter.Gun)\n  end\nend\n```","ref":"Tesla.Adapter.Gun.html#module-examples","title":"Examples - Tesla.Adapter.Gun","type":"module"},{"doc":"- `:timeout` - Time, while process, will wait for gun messages.\n\n- `:body_as` - What will be returned in `%Tesla.Env{}` body key. Possible values:\n    - `:plain` - as binary (default).\n    - `:stream` - as stream.\n        If you don't want to close connection (because you want to reuse it later)\n        pass `close_conn: false` in adapter opts.\n    - `:chunks` - as chunks.\n        You can get response body in chunks using `Tesla.Adapter.Gun.read_chunk/3` function.\n\n    Processing of the chunks and checking body size must be done by yourself.\n    Example of processing function is in `test/tesla/adapter/gun_test.exs` - `Tesla.Adapter.GunTest.read_body/4`.\n    If you don't need connection later don't forget to close it with `Tesla.Adapter.Gun.close/1`.\n\n- `:max_body` - Max response body size in bytes.\n    Works only with `body_as: :plain`, with other settings you need to check response body size by yourself.\n\n- `:conn` - Opened connection pid with gun. Is used for reusing gun connections.\n\n- `:close_conn` - Close connection or not after receiving full response body.\n    Is used for reusing gun connections. Defaults to `true`.\n\n- `:certificates_verification` - Add SSL certificates verification.\n    [erlang-certifi](https://github.com/certifi/erlang-certifi)\n    [ssl_verify_fun.erl](https://github.com/deadtrickster/ssl_verify_fun.erl)\n\n- `:proxy` - Proxy for requests.\n    **Socks proxy are supported from gun >= 2.0**.\n    Examples: `{'localhost', 1234}`, `{{127, 0, 0, 1}, 1234}`, `{:socks5, 'localhost', 1234}`.\n\n  **NOTE:** By default GUN uses TLS as transport if the specified port is 443,\n    if TLS is required for proxy connection on another port please specify transport\n    using the Gun options below otherwise tcp will be used.\n\n- `:proxy_auth` - Auth to be passed along with the proxy opt.\n    Supports Basic auth for regular and Socks proxy.\n    Format: `{proxy_username, proxy_password}`.\n\n## [Gun options](https://ninenines.eu/docs/en/gun/1.3/manual/gun/)\n\n- `:connect_timeout` - Connection timeout.\n\n- `:http_opts` - Options specific to the HTTP protocol.\n\n- `:http2_opts` -  Options specific to the HTTP/2 protocol.\n\n- `:protocols` - Ordered list of preferred protocols.\n    Defaults: `[:http2, :http]`- for :tls, `[:http]` - for :tcp.\n\n- `:trace` - Whether to enable dbg tracing of the connection process.\n    Should only be used during debugging. Default: false.\n\n- `:transport` - Whether to use TLS or plain TCP.\n    The default varies depending on the port used.\n    Port 443 defaults to tls. All other ports default to tcp.\n\n- `:transport_opts` - Transport options.\n    They are TCP options or TLS options depending on the selected transport.\n    Default: `[]`. Gun version: 1.3.\n\n- `:tls_opts` - TLS transport options.\n    Default: `[]`. Gun from master branch.\n\n- `:tcp_opts` - TCP transport options.\n    Default: `[]`. Gun from master branch.\n\n- `:socks_opts` - Options for socks.\n    Default: `[]`. Gun from master branch.\n\n- `:ws_opts` - Options specific to the Websocket protocol. Default: `%{}`.\n\n    - `:compress` - Whether to enable permessage-deflate compression.\n        This does not guarantee that compression will be used as it is the server\n        that ultimately decides. Defaults to false.\n\n    - `:protocols` - A non-empty list enables Websocket protocol negotiation.\n        The list of protocols will be sent in the sec-websocket-protocol request header.\n        The handler module interface is currently undocumented and must be set to `gun_ws_h`.","ref":"Tesla.Adapter.Gun.html#module-adapter-specific-options","title":"Adapter specific options - Tesla.Adapter.Gun","type":"module"},{"doc":"Brutally close the `gun` connection.","ref":"Tesla.Adapter.Gun.html#close/1","title":"Tesla.Adapter.Gun.close/1","type":"function"},{"doc":"Reads chunk of the response body.\n\nReturns `{:fin, binary()}` if all body received, otherwise returns `{:nofin, binary()}`.","ref":"Tesla.Adapter.Gun.html#read_chunk/3","title":"Tesla.Adapter.Gun.read_chunk/3","type":"function"},{"doc":"Adapter for [hackney](https://github.com/benoitc/hackney).\n\nRemember to add `{:hackney, \"~> 1.13\"}` to dependencies (and `:hackney` to applications in `mix.exs`)\nAlso, you need to recompile tesla after adding `:hackney` dependency:\n\n```shell\nmix deps.clean tesla\nmix deps.compile tesla\n```","ref":"Tesla.Adapter.Hackney.html","title":"Tesla.Adapter.Hackney","type":"module"},{"doc":"```elixir\n# set globally in config/config.exs\nconfig :tesla, :adapter, Tesla.Adapter.Hackney\n\n# set per module\ndefmodule MyClient do\n  def client do\n    Tesla.client([], Tesla.Adapter.Hackney)\n  end\nend\n```","ref":"Tesla.Adapter.Hackney.html#module-examples","title":"Examples - Tesla.Adapter.Hackney","type":"module"},{"doc":"- `:max_body` - Max response body size in bytes. Actual response may be bigger because hackney stops after the last chunk that surpasses `:max_body`.","ref":"Tesla.Adapter.Hackney.html#module-adapter-specific-options","title":"Adapter specific options - Tesla.Adapter.Hackney","type":"module"},{"doc":"Adapter for [httpc](http://erlang.org/doc/man/httpc.html).\n\nThis is the default adapter.\n\n**NOTE** Tesla overrides default autoredirect value with false to ensure\nconsistency between adapters","ref":"Tesla.Adapter.Httpc.html","title":"Tesla.Adapter.Httpc","type":"module"},{"doc":"Adapter for [ibrowse](https://github.com/cmullaparthi/ibrowse).\n\nRemember to add `{:ibrowse, \"~> 4.2\"}` to dependencies (and `:ibrowse` to applications in `mix.exs`)\nAlso, you need to recompile tesla after adding `:ibrowse` dependency:\n\n```elixir\nmix deps.clean tesla\nmix deps.compile tesla\n```","ref":"Tesla.Adapter.Ibrowse.html","title":"Tesla.Adapter.Ibrowse","type":"module"},{"doc":"```elixir\n# set globally in config/config.exs\nconfig :tesla, :adapter, Tesla.Adapter.Ibrowse\n\n# set per module\ndefmodule MyClient do\n  def client do\n    Tesla.client([], Tesla.Adapter.Ibrowse)\n  end\nend\n```","ref":"Tesla.Adapter.Ibrowse.html#module-examples","title":"Examples - Tesla.Adapter.Ibrowse","type":"module"},{"doc":"Adapter for [mint](https://github.com/elixir-mint/mint).\n\n**NOTE:** The minimum supported Elixir version for mint is 1.5.0\n\nRemember to add `{:mint, \"~> 1.0\"}` and `{:castore, \"~> 0.1\"}` to dependencies.\nAlso, you need to recompile tesla after adding `:mint` dependency:\n\n```shell\nmix deps.clean tesla\nmix deps.compile tesla\n```","ref":"Tesla.Adapter.Mint.html","title":"Tesla.Adapter.Mint","type":"module"},{"doc":"```elixir\n# set globally in config/config.exs\nconfig :tesla, :adapter, Tesla.Adapter.Mint\n# set per module\ndefmodule MyClient do\n  def client do\n    Tesla.client([], Tesla.Adapter.Mint)\n  end\nend\n\n# set global custom cacertfile\nconfig :tesla, adapter: {Tesla.Adapter.Mint, cacert: [\"path_to_cacert\"]}\n```","ref":"Tesla.Adapter.Mint.html#module-examples","title":"Examples - Tesla.Adapter.Mint","type":"module"},{"doc":"- `:timeout` - Time in milliseconds, while process, will wait for mint messages. Defaults to `2_000`.\n- `:body_as` - What will be returned in `%Tesla.Env{}` body key. Possible values - `:plain`, `:stream`, `:chunks`. Defaults to `:plain`.\n  - `:plain` - as binary.\n  - `:stream` - as stream. If you don't want to close connection (because you want to reuse it later) pass `close_conn: false` in adapter opts.\n  - `:chunks` - as chunks. You can get response body in chunks using `Tesla.Adapter.Mint.read_chunk/3` function.\n  Processing of the chunks and checking body size must be done by yourself. Example of processing function is in `test/tesla/adapter/mint_test.exs` - `Tesla.Adapter.MintTest.read_body/4`. If you don't need connection later don't forget to close it with `Tesla.Adapter.Mint.close/1`.\n- `:max_body` - Max response body size in bytes. Works only with `body_as: :plain`, with other settings you need to check response body size by yourself.\n- `:conn` - Opened connection with mint. Is used for reusing mint connections.\n- `:original` - Original host with port, for which reused connection was open. Needed for `Tesla.Middleware.FollowRedirects`. Otherwise adapter will use connection for another open host.\n- `:close_conn` - Close connection or not after receiving full response body. Is used for reusing mint connections. Defaults to `true`.\n- `:proxy` - Proxy settings. E.g.: `{:http, \"localhost\", 8888, []}`, `{:http, \"127.0.0.1\", 8888, []}`\n- `:transport_opts` - Keyword list of HTTP or HTTPS options passed into `:gen_tcp` or `:ssl` respectively by mint. See [mint's docs on `transport_opts`](https://hexdocs.pm/mint/Mint.HTTP.html#connect/4-transport-options).","ref":"Tesla.Adapter.Mint.html#module-adapter-specific-options","title":"Adapter specific options: - Tesla.Adapter.Mint","type":"module"},{"doc":"Closes mint connection.","ref":"Tesla.Adapter.Mint.html#close/1","title":"Tesla.Adapter.Mint.close/1","type":"function"},{"doc":"Reads chunk of the response body.\nReturns `{:fin, HTTP.t(), binary()}` if all body received, otherwise returns `{:nofin, HTTP.t(), binary()}`.","ref":"Tesla.Adapter.Mint.html#read_chunk/3","title":"Tesla.Adapter.Mint.read_chunk/3","type":"function"},{"doc":"Set base URL for all requests.\n\nThe base URL will be prepended to request path/URL only\nif it does not include http(s).","ref":"Tesla.Middleware.BaseUrl.html","title":"Tesla.Middleware.BaseUrl","type":"module"},{"doc":"```elixir\ndefmodule MyClient do\n  def client do\n    Tesla.client([\n      {Tesla.Middleware.BaseUrl, \"https://example.com/foo\"}\n    ])\n  end\nend\n\nclient = MyClient.client()\n\nTesla.get(client, \"/path\")\n# equals to GET https://example.com/foo/path\n\nTesla.get(client, \"path\")\n# equals to GET https://example.com/foo/path\n\nTesla.get(client, \"\")\n# equals to GET https://example.com/foo\n\nTesla.get(client, \"http://example.com/bar\")\n# equals to GET http://example.com/bar\n```","ref":"Tesla.Middleware.BaseUrl.html#module-examples","title":"Examples - Tesla.Middleware.BaseUrl","type":"module"},{"doc":"Basic authentication middleware.\n\n[Wiki on the topic](https://en.wikipedia.org/wiki/Basic_access_authentication)","ref":"Tesla.Middleware.BasicAuth.html","title":"Tesla.Middleware.BasicAuth","type":"module"},{"doc":"```elixir\ndefmodule MyClient do\n  def client(username, password, opts \\ %{}) do\n    Tesla.client([\n      {Tesla.Middleware.BasicAuth,\n        Map.merge(%{username: username, password: password}, opts)}\n    ])\n  end\nend\n```","ref":"Tesla.Middleware.BasicAuth.html#module-examples","title":"Examples - Tesla.Middleware.BasicAuth","type":"module"},{"doc":"- `:username` - username (defaults to `\"\"`)\n- `:password` - password (defaults to `\"\"`)","ref":"Tesla.Middleware.BasicAuth.html#module-options","title":"Options - Tesla.Middleware.BasicAuth","type":"module"},{"doc":"Bearer authentication middleware.\n\nAdds a `{\"authorization\", \"Bearer  \"}` header.","ref":"Tesla.Middleware.BearerAuth.html","title":"Tesla.Middleware.BearerAuth","type":"module"},{"doc":"```\ndefmodule MyClient do\n  def new(token) do\n    Tesla.client([\n      {Tesla.Middleware.BearerAuth, token: token}\n    ])\n  end\nend\n```","ref":"Tesla.Middleware.BearerAuth.html#module-examples","title":"Examples - Tesla.Middleware.BearerAuth","type":"module"},{"doc":"- `:token` - token (defaults to `\"\"`)","ref":"Tesla.Middleware.BearerAuth.html#module-options","title":"Options - Tesla.Middleware.BearerAuth","type":"module"},{"doc":"Only compress request.\n\nSee `Tesla.Middleware.Compression` for options.","ref":"Tesla.Middleware.CompressRequest.html","title":"Tesla.Middleware.CompressRequest","type":"module"},{"doc":"Compress requests and decompress responses.\n\nSupports \"gzip\" and \"deflate\" encodings using Erlang's built-in `:zlib` module.","ref":"Tesla.Middleware.Compression.html","title":"Tesla.Middleware.Compression","type":"module"},{"doc":"```elixir\ndefmodule MyClient do\n  def client do\n    Tesla.client([\n      {Tesla.Middleware.Compression, format: \"gzip\"}\n    ])\n  end\nend\n```","ref":"Tesla.Middleware.Compression.html#module-examples","title":"Examples - Tesla.Middleware.Compression","type":"module"},{"doc":"- `:format` - request compression format, `\"gzip\"` (default) or `\"deflate\"`","ref":"Tesla.Middleware.Compression.html#module-options","title":"Options - Tesla.Middleware.Compression","type":"module"},{"doc":"Compress request.\n\nIt is used by `Tesla.Middleware.CompressRequest`.","ref":"Tesla.Middleware.Compression.html#compress/2","title":"Tesla.Middleware.Compression.compress/2","type":"function"},{"doc":"Decompress response.\n\nIt is used by `Tesla.Middleware.DecompressResponse`.","ref":"Tesla.Middleware.Compression.html#decompress/1","title":"Tesla.Middleware.Compression.decompress/1","type":"function"},{"doc":"","ref":"Tesla.Middleware.DecodeFormUrlencoded.html","title":"Tesla.Middleware.DecodeFormUrlencoded","type":"module"},{"doc":"Decodes response body as JSON.\n\nOnly decodes the body if the `Content-Type` header suggests\nthat the body is JSON.","ref":"Tesla.Middleware.DecodeJson.html","title":"Tesla.Middleware.DecodeJson","type":"module"},{"doc":"","ref":"Tesla.Middleware.DecodeMessagePack.html","title":"Tesla.Middleware.DecodeMessagePack","type":"module"},{"doc":"","ref":"Tesla.Middleware.DecodeMessagePack.html#call/3","title":"Tesla.Middleware.DecodeMessagePack.call/3","type":"function"},{"doc":"Decode `Link` Hypermedia HTTP header into `opts[:rels]` field in response.","ref":"Tesla.Middleware.DecodeRels.html","title":"Tesla.Middleware.DecodeRels","type":"module"},{"doc":"```elixir\ndefmodule MyClient do\n  def client do\n    Tesla.client([Tesla.Middleware.DecodeRels])\n  end\nend\n\nclient = MyClient.client()\n\nenv = Tesla.get(client, \"/...\")\n\nenv.opts[:rels]\n# => %{\"Next\" => \"http://...\", \"Prev\" => \"...\"}\n```","ref":"Tesla.Middleware.DecodeRels.html#module-examples","title":"Examples - Tesla.Middleware.DecodeRels","type":"module"},{"doc":"Only decompress response.\n\nSee `Tesla.Middleware.Compression` for options.","ref":"Tesla.Middleware.DecompressResponse.html","title":"Tesla.Middleware.DecompressResponse","type":"module"},{"doc":"Digest access authentication middleware.\n\n[Wiki on the topic](https://en.wikipedia.org/wiki/Digest_access_authentication)\n\n**NOTE**: Currently the implementation is incomplete and works only for MD5 algorithm\nand auth \"quality of protection\" (qop).","ref":"Tesla.Middleware.DigestAuth.html","title":"Tesla.Middleware.DigestAuth","type":"module"},{"doc":"```\ndefmodule MyClient do\n  def client(username, password, opts \\ %{}) do\n    Tesla.client([\n      {Tesla.Middleware.DigestAuth, Map.merge(%{username: username, password: password}, opts)}\n    ])\n  end\nend\n```","ref":"Tesla.Middleware.DigestAuth.html#module-examples","title":"Examples - Tesla.Middleware.DigestAuth","type":"module"},{"doc":"- `:username` - username (defaults to `\"\"`)\n- `:password` - password (defaults to `\"\"`)\n- `:cnonce_fn` - custom function generating client nonce (defaults to `&Tesla.Middleware.DigestAuth.cnonce/0`)\n- `:nc` - nonce counter (defaults to `\"00000000\"`)","ref":"Tesla.Middleware.DigestAuth.html#module-options","title":"Options - Tesla.Middleware.DigestAuth","type":"module"},{"doc":"","ref":"Tesla.Middleware.EncodeFormUrlencoded.html","title":"Tesla.Middleware.EncodeFormUrlencoded","type":"module"},{"doc":"Encodes request body as JSON.","ref":"Tesla.Middleware.EncodeJson.html","title":"Tesla.Middleware.EncodeJson","type":"module"},{"doc":"","ref":"Tesla.Middleware.EncodeMessagePack.html","title":"Tesla.Middleware.EncodeMessagePack","type":"module"},{"doc":"","ref":"Tesla.Middleware.EncodeMessagePack.html#call/3","title":"Tesla.Middleware.EncodeMessagePack.call/3","type":"function"},{"doc":"Follow HTTP 3xx redirects.","ref":"Tesla.Middleware.FollowRedirects.html","title":"Tesla.Middleware.FollowRedirects","type":"module"},{"doc":"```elixir\ndefmodule MyClient do\n  def client do\n  # defaults to 5\n    Tesla.client([\n      {Tesla.Middleware.FollowRedirects, max_redirects: 3}\n    ])\n  end\nend\n```","ref":"Tesla.Middleware.FollowRedirects.html#module-examples","title":"Examples - Tesla.Middleware.FollowRedirects","type":"module"},{"doc":"- `:max_redirects` - limit number of redirects (default: `5`)","ref":"Tesla.Middleware.FollowRedirects.html#module-options","title":"Options - Tesla.Middleware.FollowRedirects","type":"module"},{"doc":"Send request body as `application/x-www-form-urlencoded`.\n\nPerforms encoding of `body` from a `Map` such as `%{\"foo\" => \"bar\"}` into\nURL-encoded data.\n\nPerforms decoding of the response into a map when urlencoded and content-type\nis `application/x-www-form-urlencoded`, so `\"foo=bar\"` becomes\n`%{\"foo\" => \"bar\"}`.","ref":"Tesla.Middleware.FormUrlencoded.html","title":"Tesla.Middleware.FormUrlencoded","type":"module"},{"doc":"```elixir\ndefmodule Myclient do\n  def client do\n    Tesla.client([\n      {Tesla.Middleware.FormUrlencoded,\n        encode: &Plug.Conn.Query.encode/1,\n        decode: &Plug.Conn.Query.decode/1}\n    ])\n  end\nend\n\nclient = Myclient.client()\nMyclient.post(client, \"/url\", %{key: :value})\n```","ref":"Tesla.Middleware.FormUrlencoded.html#module-examples","title":"Examples - Tesla.Middleware.FormUrlencoded","type":"module"},{"doc":"- `:decode` - decoding function, defaults to `URI.decode_query/1`\n- `:encode` - encoding function, defaults to `URI.encode_query/1`","ref":"Tesla.Middleware.FormUrlencoded.html#module-options","title":"Options - Tesla.Middleware.FormUrlencoded","type":"module"},{"doc":"Natively, nested maps are not supported in the body, so\n`%{\"foo\" => %{\"bar\" => \"baz\"}}` won't be encoded and raise an error.\nSupport for this specific case is obtained by configuring the middleware to\nencode (and decode) with `Plug.Conn.Query`\n\n```elixir\ndefmodule Myclient do\n  def client do\n    Tesla.client([\n      {Tesla.Middleware.FormUrlencoded,\n        encode: &Plug.Conn.Query.encode/1,\n        decode: &Plug.Conn.Query.decode/1}\n    ])\n  end\nend\n\nclient = Myclient.client()\nMyclient.post(client, \"/url\", %{key: %{nested: \"value\"}})\n```","ref":"Tesla.Middleware.FormUrlencoded.html#module-nested-maps","title":"Nested Maps - Tesla.Middleware.FormUrlencoded","type":"module"},{"doc":"Decode response body as querystring.\n\nIt is used by `Tesla.Middleware.DecodeFormUrlencoded`.","ref":"Tesla.Middleware.FormUrlencoded.html#decode/2","title":"Tesla.Middleware.FormUrlencoded.decode/2","type":"function"},{"doc":"Encode response body as querystring.\n\nIt is used by `Tesla.Middleware.EncodeFormUrlencoded`.","ref":"Tesla.Middleware.FormUrlencoded.html#encode/2","title":"Tesla.Middleware.FormUrlencoded.encode/2","type":"function"},{"doc":"Circuit Breaker middleware using [fuse](https://github.com/jlouis/fuse).\n\nRemember to add `{:fuse, \"~> 2.4\"}` to dependencies (and `:fuse` to applications in `mix.exs`)\nAlso, you need to recompile tesla after adding `:fuse` dependency:\n\n```\nmix deps.clean tesla\nmix deps.compile tesla\n```","ref":"Tesla.Middleware.Fuse.html","title":"Tesla.Middleware.Fuse","type":"module"},{"doc":"```elixir\ndefmodule MyClient do\n  def client do\n    Tesla.client([\n      {Tesla.Middleware.Fuse,\n        opts: {{:standard, 2, 10_000}, {:reset, 60_000}},\n        keep_original_error: true,\n        should_melt: fn\n          {:ok, %{status: status}} when status in [428, 500, 504] -> true\n          {:ok, _} -> false\n          {:error, _} -> true\n        end,\n        mode: :sync}\n    ])\n  end\nend\n```","ref":"Tesla.Middleware.Fuse.html#module-examples","title":"Examples - Tesla.Middleware.Fuse","type":"module"},{"doc":"- `:name` - fuse name (defaults to module name)\n- `:opts` - fuse options (see fuse docs for reference)\n- `:keep_original_error` - boolean to indicate if, in case of melting (based on `should_melt`), it should return the upstream's error or the fixed one `{:error, unavailable}`.\nIt's false by default, but it will be true in `2.0.0` version\n- `:should_melt` - function to determine if response should melt the fuse\n- `:mode` - how to query the fuse, which has two values:\n  - `:sync` - queries are serialized through the `:fuse_server` process (the default)\n  - `:async_dirty` - queries check the fuse state directly, but may not account for recent melts or resets","ref":"Tesla.Middleware.Fuse.html#module-options","title":"Options - Tesla.Middleware.Fuse","type":"module"},{"doc":"fuse library uses [SASL (System Architecture Support Libraries)](http://erlang.org/doc/man/sasl_app.html).\n\nYou can disable its logger output using:\n\n```elixir\nconfig :sasl, sasl_error_logger: :false\n```\n\nRead more at [jlouis/fuse#32](https://github.com/jlouis/fuse/issues/32) and [jlouis/fuse#19](https://github.com/jlouis/fuse/issues/19).","ref":"Tesla.Middleware.Fuse.html#module-sasl-logger","title":"SASL logger - Tesla.Middleware.Fuse","type":"module"},{"doc":"Set default headers for all requests","ref":"Tesla.Middleware.Headers.html","title":"Tesla.Middleware.Headers","type":"module"},{"doc":"```elixir\ndefmodule Myclient do\n  def client do\n    Tesla.client([\n      {Tesla.Middleware.Headers, [{\"user-agent\", \"Tesla\"}]}\n    ])\n  end\nend\n```","ref":"Tesla.Middleware.Headers.html#module-examples","title":"Examples - Tesla.Middleware.Headers","type":"module"},{"doc":"Encode requests and decode responses as JSON.\n\nThis middleware requires [Jason](https://hex.pm/packages/jason) (or other engine) as dependency.\n\nRemember to add `{:jason, \">= 1.0\"}` to dependencies.\nAlso, you need to recompile Tesla after adding `:jason` dependency:\n\n```\nmix deps.clean tesla\nmix deps.compile tesla\n```\n\n> #### Using built-in `JSON` from Elixir 1.18 {: .info}\n>\n> This middleware supports the built-in `JSON` module introduced in ELixir 1.18, but for historical\n> reasons is it not the default. To use it, set it as the `:engine`:\n>\n>     {Tesla.Middleware.JSON, engine: JSON}\n>\n> For more advanced usage using custom encoders/decodes, provide the `:encode` and `:decode` anonymous functions instead.\n\nIf you only need to encode the request body or decode the response body,\nyou can use `Tesla.Middleware.EncodeJson` or `Tesla.Middleware.DecodeJson` directly instead.","ref":"Tesla.Middleware.JSON.html","title":"Tesla.Middleware.JSON","type":"module"},{"doc":"```\ndefmodule MyClient do\n  def client do\n    Tesla.client([\n      # use jason engine\n      Tesla.Middleware.JSON,\n      # or\n      {Tesla.Middleware.JSON, engine: JSON}\n      # or\n      {Tesla.Middleware.JSON, engine: JSX, engine_opts: [strict: [:comments]]},\n      # or\n      {Tesla.Middleware.JSON, engine: Poison, engine_opts: [keys: :atoms]},\n      # or\n      {Tesla.Middleware.JSON, decode: &JSX.decode/1, encode: &JSX.encode/1}\n    ])\n  end\nend\n```","ref":"Tesla.Middleware.JSON.html#module-examples","title":"Examples - Tesla.Middleware.JSON","type":"module"},{"doc":"- `:decode` - decoding function\n- `:encode` - encoding function\n- `:encode_content_type` - content-type to be used in request header\n- `:engine` - encode/decode engine, e.g `JSON`, `Jason`, `Poison` or `JSX`  (defaults to Jason)\n- `:engine_opts` - optional engine options\n- `:decode_content_types` - list of additional decodable content-types","ref":"Tesla.Middleware.JSON.html#module-options","title":"Options - Tesla.Middleware.JSON","type":"module"},{"doc":"Decode response body as JSON.\n\nIt is used by `Tesla.Middleware.DecodeJson`.","ref":"Tesla.Middleware.JSON.html#decode/2","title":"Tesla.Middleware.JSON.decode/2","type":"function"},{"doc":"Encode request body as JSON.\n\nIt is used by `Tesla.Middleware.EncodeJson`.","ref":"Tesla.Middleware.JSON.html#encode/2","title":"Tesla.Middleware.JSON.encode/2","type":"function"},{"doc":"Store request URL, body, and headers into `:opts`.","ref":"Tesla.Middleware.KeepRequest.html","title":"Tesla.Middleware.KeepRequest","type":"module"},{"doc":"```\ndefmodule MyClient do\n  def client do\n    Tesla.client([\n      Tesla.Middleware.KeepRequest,\n      Tesla.Middleware.PathParams\n    ])\n  end\nend\n\nclient = MyClient.client()\n{:ok, env} = Tesla.post(client, \"/users/:user_id\", \"request-data\", opts: [path_params: [user_id: \"1234\"]])\n\nenv.body\n# => \"response-data\"\n\nenv.opts[:req_body]\n# => \"request-data\"\n\nenv.opts[:req_headers]\n# => [{\"request-headers\", \"are-safe\"}, ...]\n\nenv.opts[:req_url]\n# => \"http://localhost:8000/users/:user_id\n```","ref":"Tesla.Middleware.KeepRequest.html#module-examples","title":"Examples - Tesla.Middleware.KeepRequest","type":"module"},{"doc":"In practice, you would combine `Tesla.Middleware.KeepRequest`, `Tesla.Middleware.PathParams`, and\n`Tesla.Middleware.Telemetry` to observe the request and response data.\nKeep in mind that the request order matters. Make sure to put `Tesla.Middleware.KeepRequest` before\n`Tesla.Middleware.PathParams` to make sure that the request data is stored before the path parameters are replaced.\nWhile keeping in mind that this is an application-specific concern, this is the overall recommendation.","ref":"Tesla.Middleware.KeepRequest.html#module-observability","title":"Observability - Tesla.Middleware.KeepRequest","type":"module"},{"doc":"Log requests using Elixir's Logger.\n\nWith the default settings it logs request method, URL, response status, and\ntime taken in milliseconds.","ref":"Tesla.Middleware.Logger.html","title":"Tesla.Middleware.Logger","type":"module"},{"doc":"```elixir\ndefmodule MyClient do\n  def client do\n    Tesla.client([Tesla.Middleware.Logger])\n  end\nend\n```","ref":"Tesla.Middleware.Logger.html#module-examples","title":"Examples - Tesla.Middleware.Logger","type":"module"},{"doc":"- `:level` - custom function for calculating log level or atom for fixed level (see below)\n- `:log_level` - (deprecated) custom function for calculating log level (see below)\n- `:filter_headers` - sanitizes sensitive headers before logging in debug mode (see below)\n- `:debug` - use `Logger.debug/2` to log request/response details\n- `:format` - custom string template or function for log message (see below)","ref":"Tesla.Middleware.Logger.html#module-options","title":"Options - Tesla.Middleware.Logger","type":"module"},{"doc":"The default log format is `\"$method $url -> $status ($time ms)\"`\nwhich shows in logs like:\n\n```elixir\n2018-03-25 18:32:40.397 [info]  GET https://bitebot.io -> 200 (88.074 ms)\n```\n\nIt can be changed globally with config:\n\n```elixir\nconfig :tesla, Tesla.Middleware.Logger, format: \"$method $url ====> $status / time=$time\"\n```\n\nOr you can customize this setting by providing your own `format` function:\n\n```elixir\ndefmodule MyClient do\n  def client do\n    Tesla.client([\n      {Tesla.Middleware.Logger, format: &my_format/3}\n    ])\n  end\n\n  def my_format(request, response, time) do\n    \"request=#{inspect(request)} response=#{inspect(response)} time=#{time}\\n\"\n  end\nend\n```","ref":"Tesla.Middleware.Logger.html#module-custom-log-format","title":"Custom log format - Tesla.Middleware.Logger","type":"module"},{"doc":"By default, the following log levels will be used:\n\n- `:error` - for errors, 5xx and 4xx responses\n- `:warn` or `:warning` - for 3xx responses\n- `:info` - for 2xx responses\n\nYou can customize this setting by providing your own level function that accepts\nboth success and error cases:\n\n```elixir\ndefmodule MyClient do\n  def client do\n    Tesla.client([\n      {Tesla.Middleware.Logger, level: &my_level/1}\n    ])\n  end\n\n  def my_level({:ok, env}) do\n    case env.status do\n      404 -> :info\n      _ -> :default\n    end\n  end\n\n  def my_level({:error, _reason}) do\n    :error\n  end\nend\n```\n\nOr provide a fixed log level:\n\n```elixir\ndefmodule MyClient do\n  def client do\n    Tesla.client([\n      {Tesla.Middleware.Logger, level: :debug}\n    ])\n  end\nend\n```\n\nYou can also use the deprecated `log_level` option (will show a deprecation warning):\n\n```elixir\ndefmodule MyClient do\n  def client do\n    Tesla.client([\n      {Tesla.Middleware.Logger, log_level: &my_log_level/1}\n    ])\n  end\n\n  def my_log_level(env) do\n    case env.status do\n      404 -> :info\n      _ -> :default\n    end\n  end\nend\n```\n\nTo disable the deprecation warning for `:log_level`, add this to your config:\n\n```elixir\n# config/config.exs\nconfig :tesla, disable_log_level_warning: true\n```","ref":"Tesla.Middleware.Logger.html#module-custom-log-levels","title":"Custom log levels - Tesla.Middleware.Logger","type":"module"},{"doc":"`Tesla` will use `Logger.debug/2` to log request & response details using\nthe `:debug` option. It will require to set the `Logger` log level to `:debug`\nin your configuration, example:\n\n```elixir\n# config/dev.exs\nconfig :logger, level: :debug\n```\n\nIf you want to disable detailed request/response logging but keep the\n`:debug` log level (i.e. in development) you can set `debug: false` in your\nconfig:\n\n```elixir\n# config/dev.local.exs\nconfig :tesla, Tesla.Middleware.Logger, debug: false\n```\n\nNote that the logging configuration is evaluated at compile time,\nso Tesla must be recompiled for the configuration to take effect:\n\n```shell\nmix deps.clean --build tesla\nmix deps.compile tesla\n```\n\nIn order to be able to set `:debug` at runtime we can\npass it as a option to the middleware at runtime.\n\n```elixir\ndef client do\n  middleware = [\n    # ...\n    {Tesla.Middleware.Logger, debug: false}\n  ]\n\n  Tesla.client(middleware)\nend\n```","ref":"Tesla.Middleware.Logger.html#module-logger-debug-output","title":"Logger Debug output - Tesla.Middleware.Logger","type":"module"},{"doc":"To sanitize sensitive headers such as `authorization` in\ndebug logs, add them to the `:filter_headers` option.\n`:filter_headers` expects a list of header names as strings.\n\n```elixir\n# config/dev.local.exs\nconfig :tesla, Tesla.Middleware.Logger,\n  filter_headers: [\"authorization\"]\n```","ref":"Tesla.Middleware.Logger.html#module-filter-headers","title":"Filter headers - Tesla.Middleware.Logger","type":"module"},{"doc":"","ref":"Tesla.Middleware.Logger.html#default_log_level/1","title":"Tesla.Middleware.Logger.default_log_level/1","type":"function"},{"doc":"","ref":"Tesla.Middleware.Logger.html#t:log_level/0","title":"Tesla.Middleware.Logger.log_level/0","type":"type"},{"doc":"Encode requests and decode responses as MessagePack.\n\nThis middleware requires [Msgpax](https://hex.pm/packages/msgpax) as dependency.\n\nRemember to add `{:msgpax, \">= 2.3.0\"}` to dependencies.\nAlso, you need to recompile Tesla after adding `:msgpax` dependency:\n\n```shell\nmix deps.clean tesla\nmix deps.compile tesla\n```","ref":"Tesla.Middleware.MessagePack.html","title":"Tesla.Middleware.MessagePack","type":"module"},{"doc":"```elixir\ndefmodule MyClient do\n  def client do\n    Tesla.client([\n      Tesla.Middleware.MessagePack,\n      # or\n      {Tesla.Middleware.MessagePack, engine_opts: [binary: true]},\n      # or\n      {Tesla.Middleware.MessagePack, decode: &Custom.decode/1, encode: &Custom.encode/1}\n    ])\n  end\nend\n```","ref":"Tesla.Middleware.MessagePack.html#module-examples","title":"Examples - Tesla.Middleware.MessagePack","type":"module"},{"doc":"- `:decode` - decoding function\n- `:encode` - encoding function\n- `:encode_content_type` - content-type to be used in request header\n- `:decode_content_types` - list of additional decodable content-types\n- `:engine_opts` - optional engine options","ref":"Tesla.Middleware.MessagePack.html#module-options","title":"Options - Tesla.Middleware.MessagePack","type":"module"},{"doc":"Decode response body as MessagePack.\n\nIt is used by `Tesla.Middleware.DecodeMessagePack`.","ref":"Tesla.Middleware.MessagePack.html#decode/2","title":"Tesla.Middleware.MessagePack.decode/2","type":"function"},{"doc":"Encode request body as MessagePack.\n\nIt is used by `Tesla.Middleware.EncodeMessagePack`.","ref":"Tesla.Middleware.MessagePack.html#encode/2","title":"Tesla.Middleware.MessagePack.encode/2","type":"function"},{"doc":"Middleware that adds `X-HTTP-Method-Override` header with original request\nmethod and sends the request as post.\n\nUseful when there's an issue with sending non-POST request.","ref":"Tesla.Middleware.MethodOverride.html","title":"Tesla.Middleware.MethodOverride","type":"module"},{"doc":"```elixir\ndefmodule MyClient do\n  def client do\n    Tesla.client([Tesla.Middleware.MethodOverride])\n  end\nend\n```","ref":"Tesla.Middleware.MethodOverride.html#module-examples","title":"Examples - Tesla.Middleware.MethodOverride","type":"module"},{"doc":"- `:override` - list of HTTP methods that should be overridden, everything except `:get` and `:post` if not specified","ref":"Tesla.Middleware.MethodOverride.html#module-options","title":"Options - Tesla.Middleware.MethodOverride","type":"module"},{"doc":"Set default opts for all requests.","ref":"Tesla.Middleware.Opts.html","title":"Tesla.Middleware.Opts","type":"module"},{"doc":"```elixir\ndefmodule MyClient do\n  def client do\n    Tesla.client([\n      {Tesla.Middleware.Opts, [some: \"option\"]}\n    ])\n  end\nend\n```","ref":"Tesla.Middleware.Opts.html#module-examples","title":"Examples - Tesla.Middleware.Opts","type":"module"},{"doc":"Use templated URLs with provided parameters in either Phoenix style (`:id`)\nor OpenAPI style (`{id}`).\n\nUseful when logging or reporting metrics per URL.","ref":"Tesla.Middleware.PathParams.html","title":"Tesla.Middleware.PathParams","type":"module"},{"doc":"Parameter values may be `t:struct/0` or must implement the `Enumerable`\nprotocol and produce `{key, value}` tuples when enumerated.","ref":"Tesla.Middleware.PathParams.html#module-parameter-values","title":"Parameter Values - Tesla.Middleware.PathParams","type":"module"},{"doc":"Phoenix style parameters may contain letters, numbers, or underscores,\nmatching this regular expression:\n\n  :[a-zA-Z][_a-zA-Z0-9]*\b\n\nOpenAPI style parameters may contain letters, numbers, underscores, or\nhyphens (`-`), matching this regular expression:\n\n  {[a-zA-Z][-_a-zA-Z0-9]*}\n\nIn either case, parameters that begin with underscores (`_`), hyphens (`-`),\nor numbers (`0-9`) are ignored and left as-is.","ref":"Tesla.Middleware.PathParams.html#module-parameter-name-restrictions","title":"Parameter Name Restrictions - Tesla.Middleware.PathParams","type":"module"},{"doc":"```elixir\ndefmodule MyClient do\n  def client do\n    Tesla.client([\n      {Tesla.Middleware.BaseUrl, \"https://api.example.com\"},\n      Tesla.Middleware.Logger,\n      Tesla.Middleware.PathParams\n    ])\n  end\n\n  def user(client, id) do\n    params = [id: id]\n    Tesla.get(client, \"/users/{id}\", opts: [path_params: params])\n  end\n\n  def posts(client, id, post_id) do\n    params = [id: id, post_id: post_id]\n    Tesla.get(client, \"/users/:id/posts/:post_id\", opts: [path_params: params])\n  end\nend\n```","ref":"Tesla.Middleware.PathParams.html#module-examples","title":"Examples - Tesla.Middleware.PathParams","type":"module"},{"doc":"Set default query params for all requests","ref":"Tesla.Middleware.Query.html","title":"Tesla.Middleware.Query","type":"module"},{"doc":"```elixir\ndefmodule MyClient do\n  def client do\n    Tesla.client([\n      {Tesla.Middleware.Query, [token: \"some-token\"]}\n    ])\n  end\nend\n```","ref":"Tesla.Middleware.Query.html#module-examples","title":"Examples - Tesla.Middleware.Query","type":"module"},{"doc":"Retry using exponential backoff and full jitter.\n\nBy defaults, this middleware only retries in the case of connection errors (`nxdomain`, `connrefused`, etc).\nApplication error checking for retry can be customized through `:should_retry` option.","ref":"Tesla.Middleware.Retry.html","title":"Tesla.Middleware.Retry","type":"module"},{"doc":"The backoff algorithm optimizes for tight bounds on completing a request successfully.\nIt does this by first calculating an exponential backoff factor based on the\nnumber of retries that have been performed.  It then multiplies this factor against\nthe base delay. The total maximum delay is found by taking the minimum of either\nthe calculated delay or the maximum delay specified. This creates an upper bound\non the maximum delay we can see.\n\nIn order to find the actual delay value we apply additive noise which is proportional\nto the current desired delay. This ensures that the actual delay is kept within\nthe expected order of magnitude, while still having some randomness, which ensures\nthat our retried requests don't \"harmonize\" making it harder for the downstream service to heal.","ref":"Tesla.Middleware.Retry.html#module-backoff-algorithm","title":"Backoff algorithm - Tesla.Middleware.Retry","type":"module"},{"doc":"```elixir\ndefmodule MyClient do\n  def client do\n    Tesla.client([\n      {Tesla.Middleware.Retry,\n        delay: 500,\n        max_retries: 10,\n        max_delay: 4_000,\n        should_retry: fn\n          {:ok, %{status: status}}, _env, _context when status in [400, 500] -> true\n          {:ok, _reason}, _env, _context -> false\n          {:error, _reason}, %Tesla.Env{method: :post}, _context -> false\n          {:error, _reason}, %Tesla.Env{method: :put}, %{retries: 2} -> false\n          {:error, _reason}, _env, _context -> true\n        end\n      }\n    ])\n  end\nend\n```","ref":"Tesla.Middleware.Retry.html#module-examples","title":"Examples - Tesla.Middleware.Retry","type":"module"},{"doc":"- `:delay` - The base delay in milliseconds (positive integer, defaults to 50)\n- `:max_retries` - maximum number of retries (non-negative integer, defaults to 5)\n- `:max_delay` - maximum delay in milliseconds (positive integer, defaults to 5000)\n- `:should_retry` - function with an arity of 1 or 3 used to determine if the request should\n    be retried the first argument is the result, the second is the env and the third is\n    the context: options + `:retries` (defaults to a match on `{:error, _reason}`)\n- `:jitter_factor` - additive noise proportionality constant\n    (float between 0 and 1, defaults to 0.2)\n- `:use_retry_after_header` - whether to use the Retry-After header to determine the minimum\n    delay before the next retry.  If the delay from the header exceeds max_delay, no further\n    retries are attempted.  Invalid Retry-After headers are ignored.\n    (boolean, defaults to false)","ref":"Tesla.Middleware.Retry.html#module-options","title":"Options - Tesla.Middleware.Retry","type":"module"},{"doc":"","ref":"Tesla.Middleware.Retry.html#retry_after_with_jitter/3","title":"Tesla.Middleware.Retry.retry_after_with_jitter/3","type":"function"},{"doc":"Decode Server Sent Events.\n\nThis middleware is mostly useful when streaming response body.","ref":"Tesla.Middleware.SSE.html","title":"Tesla.Middleware.SSE","type":"module"},{"doc":"```elixir\ndefmodule MyClient do\n  def client do\n    Tesla.client([Tesla.Middleware.SSE, only: :data])\n  end\nend\n```","ref":"Tesla.Middleware.SSE.html#module-examples","title":"Examples - Tesla.Middleware.SSE","type":"module"},{"doc":"- `:only` - keep only specified keys in event (necessary for using with `JSON` middleware)\n- `:decode_content_types` - list of additional decodable content-types","ref":"Tesla.Middleware.SSE.html#module-options","title":"Options - Tesla.Middleware.SSE","type":"module"},{"doc":"","ref":"Tesla.Middleware.SSE.html#decode/2","title":"Tesla.Middleware.SSE.decode/2","type":"function"},{"doc":"Emits events using the `:telemetry` library to expose instrumentation.","ref":"Tesla.Middleware.Telemetry.html","title":"Tesla.Middleware.Telemetry","type":"module"},{"doc":"```elixir\ndefmodule MyClient do\n  def client do\n    Tesla.client([Tesla.Middleware.Telemetry])\n  end\nend\n\n:telemetry.attach(\n  \"my-tesla-telemetry\",\n  [:tesla, :request, :stop],\n  fn event, measurements, meta, config ->\n    # Do something with the event\n  end,\n  nil\n)\n```","ref":"Tesla.Middleware.Telemetry.html#module-examples","title":"Examples - Tesla.Middleware.Telemetry","type":"module"},{"doc":"- `:metadata` - additional metadata passed to telemetry events","ref":"Tesla.Middleware.Telemetry.html#module-options","title":"Options - Tesla.Middleware.Telemetry","type":"module"},{"doc":"* `[:tesla, :request, :start]` - emitted at the beginning of the request.\n  * Measurement: `%{system_time: System.system_time()}`\n  * Metadata: `%{env: Tesla.Env.t()}`\n\n* `[:tesla, :request, :stop]` - emitted at the end of the request.\n  * Measurement: `%{duration: native_time}`\n  * Metadata: `%{env: Tesla.Env.t()} | %{env: Tesla.Env.t(), error: term()}`\n\n* `[:tesla, :request, :exception]` - emitted when an exception has been raised.\n  * Measurement: `%{duration: native_time}`\n  * Metadata: `%{env: Tesla.Env.t(), kind: Exception.kind(), reason: term(), stacktrace: Exception.stacktrace()}`","ref":"Tesla.Middleware.Telemetry.html#module-telemetry-events","title":"Telemetry Events - Tesla.Middleware.Telemetry","type":"module"},{"doc":"* `[:tesla, :request]` - This event is emitted for backwards compatibility only and should be considered deprecated.\n      This event can be disabled by setting `config :tesla, Tesla.Middleware.Telemetry, disable_legacy_event: true` in your config.\n      Be sure to run `mix deps.compile --force tesla` after changing this setting to ensure the change is picked up.\n\nPlease check the [telemetry](https://hexdocs.pm/telemetry/) for further usage.","ref":"Tesla.Middleware.Telemetry.html#module-legacy-telemetry-events","title":"Legacy Telemetry Events - Tesla.Middleware.Telemetry","type":"module"},{"doc":"Sometimes, it is useful to have access to a template URL (i.e. `\"/users/:user_id\"`) for grouping\nTelemetry events. For such cases, a combination of the `Tesla.Middleware.PathParams`,\n`Tesla.Middleware.Telemetry` and `Tesla.Middleware.KeepRequest` may be used.\n\n```elixir\ndefmodule MyClient do\n  def client do\n    Tesla.client([\n      # The KeepRequest middleware sets the template URL as a Tesla.Env.opts entry\n      # Said entry must be used because on happy-path scenarios,\n      # the Telemetry middleware will receive the Tesla.Env.url resolved by PathParams.\n      Tesla.Middleware.KeepRequest,\n      Tesla.Middleware.PathParams,\n      Tesla.Middleware.Telemetry\n    ])\n  end\nend\n\n:telemetry.attach(\n  \"my-tesla-telemetry\",\n  [:tesla, :request, :stop],\n  fn event, measurements, meta, config ->\n    path_params_template_url = meta.env.opts[:req_url]\n    # The meta.env.url key will only present the resolved URL on happy-path scenarios.\n    # Error cases will still return the original template URL.\n    path_params_resolved_url = meta.env.url\n  end,\n  nil\n)\n```\n\n> #### Order Matters {: .warning}\n> Place the `Tesla.Middleware.Telemetry` middleware as close as possible to\n> the end of the middleware stack to ensure that you are measuring the\n> actual request itself and do not lose any information about the\n> `t:Tesla.Env.t/0` due to some transformation that happens in the\n> middleware stack before reaching the `Tesla.Middleware.Telemetry`\n> middleware.","ref":"Tesla.Middleware.Telemetry.html#module-url-event-scoping-with-tesla-middleware-pathparams-and-tesla-middleware-keeprequest","title":"URL event scoping with Tesla.Middleware.PathParams and Tesla.Middleware.KeepRequest - Tesla.Middleware.Telemetry","type":"module"},{"doc":"Timeout HTTP request after X milliseconds.","ref":"Tesla.Middleware.Timeout.html","title":"Tesla.Middleware.Timeout","type":"module"},{"doc":"```elixir\ndefmodule MyClient do\n  def client do\n    Tesla.client([\n      {Tesla.Middleware.Timeout, timeout: 2_000}\n    ])\n  end\nend\n```\n\nIf you are using OpenTelemetry in your project, you may be interested in\nusing `OpentelemetryProcessPropagator.Task` to have a better integration using\nthe `task_module` option.\n\n```elixir\ndefmodule MyClient do\n  def client do\n    Tesla.client([\n      {Tesla.Middleware.Timeout, [\n        timeout: 2_000,\n        task_module: OpentelemetryProcessPropagator.Task\n      }\n    ])\n  end\nend\n```","ref":"Tesla.Middleware.Timeout.html#module-examples","title":"Examples - Tesla.Middleware.Timeout","type":"module"},{"doc":"- `:timeout` - number of milliseconds a request is allowed to take (defaults to `1000`)\n- `:task_module` - the `Task` module used to spawn tasks. Useful when you want\n  to use alternatives such as `OpentelemetryProcessPropagator.Task` from the OTEL\n  project.","ref":"Tesla.Middleware.Timeout.html#module-options","title":"Options - Tesla.Middleware.Timeout","type":"module"},{"doc":"","ref":"Tesla.Error.html","title":"Tesla.Error","type":"exception"},{"doc":"Callback implementation for  Exception.message/1 .","ref":"Tesla.Error.html#message/1","title":"Tesla.Error.message/1","type":"function"},{"doc":"","ref":"Tesla.Mock.Error.html","title":"Tesla.Mock.Error","type":"exception"},{"doc":"Callback implementation for  Exception.message/1 .","ref":"Tesla.Mock.Error.html#message/1","title":"Tesla.Mock.Error.message/1","type":"function"},{"doc":"# Tesla\n\n[![Test](https://github.com/elixir-tesla/tesla/actions/workflows/test.yml/badge.svg)](https://github.com/elixir-tesla/tesla/actions/workflows/test.yml)\n[![Hex.pm](https://img.shields.io/hexpm/v/tesla.svg)](https://hex.pm/packages/tesla)\n[![Hexdocs.pm](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/tesla/)\n[![Hex.pm](https://img.shields.io/hexpm/dt/tesla.svg)](https://hex.pm/packages/tesla)\n[![Hex.pm](https://img.shields.io/hexpm/dw/tesla.svg)](https://hex.pm/packages/tesla)\n[![codecov](https://codecov.io/gh/elixir-tesla/tesla/branch/master/graph/badge.svg)](https://codecov.io/gh/elixir-tesla/tesla)\n\n`Tesla` is an HTTP client that leverages middleware to streamline HTTP requests\nand responses over a common interface for various adapters.\n\nIt simplifies HTTP communication by providing a flexible and composable\nmiddleware stack. Developers can easily build custom API clients by stacking\nmiddleware components that handle tasks like authentication, logging, and\nretries. `Tesla` supports multiple HTTP adapters such as `Mint`, `Finch`,\n`Hackney`, etc.\n\n`Tesla` is ideal for developers who need a flexible and efficient HTTP client.\nIts ability to swap out HTTP adapters and create custom middleware pipelines\nempowers you to make different architectural decisions and build tools tailored\nto your application's needs with minimal effort.\n\nInspired by [Faraday](https://github.com/lostisland/faraday) from Ruby.","ref":"readme.html","title":"Tesla","type":"extras"},{"doc":"Add `:tesla` as dependency in `mix.exs`:\n\n```elixir\ndefp deps do\n  [\n     # or latest version\n    {:tesla, \"~> 1.11\"},\n    # optional, required by JSON middleware\n    {:jason, \"~> 1.4\"},\n    # optional, required by Mint adapter, recommended\n    {:mint, \"~> 1.0\"}\n  ]\nend\n```\n\n> #### :httpc as default Adapter {: .error}\n> The default adapter is erlang's built-in `httpc`, primarily to avoid\n> additional\n> dependencies when using `Tesla` in a new project. But it is not recommended to\n> use it in production environment as it does not validate SSL certificates\n> [among other issues](https://github.com/elixir-tesla/tesla/issues?utf8=%E2%9C%93&q=is%3Aissue+label%3Ahttpc+).\n> Instead, consider using `Mint`, `Finch`, or `Hackney` adapters.\n> We believe that such security issues should be addressed by `:httpc` itself\n> and we are not planning to fix them in `Tesla` due to backward compatibility.\n\nConfigure default adapter in `config/config.exs`.\n\n```elixir\n# config/config.exs\n\n# Make sure to install `mint` package as well, recommended\nconfig :tesla, adapter: Tesla.Adapter.Mint\n```\n\nTo make a simple GET request, run `iex -S mix` and execute:\n\n    iex> Tesla.get!(\"https://httpbin.org/get\").status\n    # => 200\n\nThat will not include any middleware and will use the global default adapter.\nCreate a client to compose middleware and reuse it across requests.\n\n    iex> client = Tesla.client([\n    ...>  {Tesla.Middleware.BaseUrl, \"https://httpbin.org/\"},\n    ...>  Tesla.Middleware.JSON,\n    ...> ])\n\n    iex> Tesla.get!(client, \"/json\").body\n    # => %{\"slideshow\" => ...}\n\nLastly, you can enforce the adapter to be used by a specific client:\n\n    iex> client = Tesla.client([], {Tesla.Adapter.Hackney, pool: :my_pool})\n\nHappy hacking!","ref":"readme.html#getting-started","title":"Getting started - Tesla","type":"extras"},{"doc":"Check out the following sections to learn more about `Tesla`:","ref":"readme.html#what-to-do-next","title":"What to do next? - Tesla","type":"extras"},{"doc":"- [Client](./guides/explanations/0.client.md)\n- [Testing](./guides/explanations/1.testing.md)\n- [Middleware](./guides/explanations/2.middleware.md)\n- [Adapter](./guides/explanations/3.adapter.md)","ref":"readme.html#explanations","title":"Explanations - Tesla","type":"extras"},{"doc":"#### Migrations\n\n- [Migrating from v0 to v1.x](./guides/howtos/migrations/v0-to-v1.md)","ref":"readme.html#howtos","title":"Howtos - Tesla","type":"extras"},{"doc":"- [General Cheatsheet](./guides/cheatsheets/general.cheatmd)\n- [Cookbook](https://github.com/elixir-tesla/tesla/wiki)\n\n#### Middleware\n\n`Tesla` is built around the concept of composable middlewares.\n\n- `Tesla.Middleware.BaseUrl` - set base URL.\n- `Tesla.Middleware.Headers` - set request headers.\n- `Tesla.Middleware.Query` - set query parameters.\n- `Tesla.Middleware.Opts` - set request options.\n- `Tesla.Middleware.FollowRedirects` - follow HTTP 3xx redirects.\n- `Tesla.Middleware.MethodOverride` - set `X-Http-Method-Override` header.\n- `Tesla.Middleware.Logger` - log requests (method, url, status, and time).\n- `Tesla.Middleware.KeepRequest` - keep request `body` and `headers`.\n- `Tesla.Middleware.PathParams` - use templated URLs.\n\n##### Formats\n\n- `Tesla.Middleware.FormUrlencoded` - URL encode POST body, useful for POSTing a\n  map/keyword list.\n- `Tesla.Middleware.JSON` - encode/decode JSON request/response body.\n- `Tesla.Middleware.Compression` - compress request/response body using\n  `gzip` and `deflate`.\n- `Tesla.Middleware.DecodeRels` - decode `Link` header into `opts[:rels]` field\n  in response.\n\n##### Auth\n\n- `Tesla.Middleware.BasicAuth` - HTTP Basic Auth.\n- `Tesla.Middleware.BearerAuth` - HTTP Bearer Auth.\n- `Tesla.Middleware.DigestAuth`] - Digest access authentication.\n\n##### Error handling\n\n- `Tesla.Middleware.Timeout` - timeout request after X milliseconds despite of\n  server response.\n- `Tesla.Middleware.Retry` - retry few times in case of connection refused.\n- `Tesla.Middleware.Fuse` - fuse circuit breaker integration.\n\n#### Adapters\n\nTesla supports multiple HTTP adapter that do the actual HTTP request processing.\n\n- `Tesla.Adapter.Httpc` - the default, built-in Erlang [httpc][0] adapter.\n- `Tesla.Adapter.Hackney` - [hackney][1], simple HTTP client in Erlang.\n- `Tesla.Adapter.Ibrowse` - [ibrowse][2], Erlang HTTP client.\n- `Tesla.Adapter.Gun` - [gun][3], HTTP/1.1, HTTP/2 and Websocket client for\n  Erlang/OTP.\n- `Tesla.Adapter.Mint` - [mint][4], Functional HTTP client for Elixir with\n  support for HTTP/1 and HTTP/2.\n- `Tesla.Adapter.Finch` - [finch][5], An HTTP client with a focus on\n  performance, built on top of [Mint][4] and [NimblePool][6].","ref":"readme.html#references","title":"References - Tesla","type":"extras"},{"doc":"- [ubots - Ultimate Productivity Made Easy with Slack](https://ubots.co/)\n\n[0]: https://erlang.org/doc/man/httpc.html\n[1]: https://github.com/benoitc/hackney\n[2]: https://github.com/cmullaparthi/ibrowse\n[3]: https://github.com/ninenines/gun\n[4]: https://github.com/elixir-mint/mint\n[5]: https://github.com/keathley/finch\n[6]: https://github.com/dashbitco/nimble_pool","ref":"readme.html#sponsors","title":"Sponsors - Tesla","type":"extras"},{"doc":"The MIT License (MIT)\n\nCopyright (c) 2015-Present Tymon Tobolski\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html","title":"License","type":"extras"},{"doc":"# Client\n\nIn Tesla, a **client** is an entity that combines middleware and an adapter,\ncreated using `Tesla.client/2`. Middleware components modify or enhance requests\nand responses—such as adding headers or handling authentication—while adapters\nhandle the underlying HTTP communication. For more details, see the sections on\n[middleware](./2.middleware.md) and [adapters](./3.adapter.md).","ref":"0-client.html","title":"Client","type":"extras"},{"doc":"A client is created using `Tesla.client/2`, which takes a list of middleware\nand an adapter.\n\n```elixir\nclient = Tesla.client([Tesla.Middleware.PathParams, Tesla.Middleware.Logger])\n```\n\nYou can then use the client to make requests:\n\n```elixir\nTesla.get(client, \"/users/123\")\n```","ref":"0-client.html#creating-a-client","title":"Creating a Client - Client","type":"extras"},{"doc":"You can pass options to middleware by registering the middleware as a tuple\nof two elements, where the first element is the middleware module and the\nsecond is the options.\n\n```elixir\nclient = Tesla.client(\n  [{Tesla.Middleware.BaseUrl, \"https://api.example.com\"}]\n)\n```","ref":"0-client.html#passing-options-to-middleware","title":"Passing Options to Middleware - Client","type":"extras"},{"doc":"By default, the global adapter is used. You can override this by passing an\nadapter to the client.\n\n```elixir\nclient = Tesla.client([], Tesla.Adapter.Mint)\n```\nYou can also pass options to the adapter.\n\n```elixir\nclient = Tesla.client([], {Tesla.Adapter.Mint, pool: :my_pool})\n```","ref":"0-client.html#passing-adapter","title":"Passing Adapter - Client","type":"extras"},{"doc":"A common approach in applications is to encapsulate client creation within a\nmodule or function that sets up standard middleware and adapter configurations.\nThis results in a single, shared client instance used throughout the\napplication. For example:\n\n```elixir\ndefmodule MyApp.ServiceName do\n  defp client do\n    middleware = [\n      {Tesla.Middleware.BaseUrl, \"https://api.service.com\"},\n      {Tesla.Middleware.BearerAuth, token: bearer_token()},\n      # Additional middleware...\n    ]\n    Tesla.client(middleware, adapter())\n  end\n\n  defp adapter do\n    Keyword.get(config(), :adapter)\n  end\n\n  defp bearer_token do\n    Keyword.fetch!(config(), :bearer_token)\n  end\n\n  defp config do\n    Application.get_env(:my_app, __MODULE__, [])\n  end\nend\n```\n\nIn this pattern, the client is constructed internally, and operations use this\nsingleton client:\n\n```elixir\ndefmodule MyApp.ServiceName do\n  def operation_name(body) do\n    url = \"/endpoint\"\n    # The client() function is called internally\n    response = Tesla.post!(client(), url, body)\n    # Process the response...\n  end\n\n  defp client do\n    # Client construction as shown earlier\n  end\nend\n```\n\nYou can then use the module to make requests without managing the client externally:\n\n```elixir\n{:ok, response} = MyApp.ServiceName.operation_name(%{key: \"value\"})\n```","ref":"0-client.html#single-client-singleton-pattern","title":"Single-Client (Singleton) Pattern - Client","type":"extras"},{"doc":"In scenarios where different configurations are needed—such as multi-tenancy\napplications or interacting with multiple services—you can modify the client\nfunction to accept configuration parameters. This allows for the creation of\nmultiple clients with varying settings:\n\n```elixir\ndefmodule MyApp.ServiceName do\n  def operation_name(client, body) do\n    url = \"/endpoint\"\n    # The client is passed as a parameter\n    response = Tesla.post!(client, url, body)\n    # Process the response...\n  end\n\n  def client(opts) do\n    middleware = [\n      {Tesla.Middleware.BaseUrl, opts[:base_url]},\n      {Tesla.Middleware.BearerAuth, token: opts[:bearer_token]},\n      # Additional middleware...\n    ]\n    Tesla.client(middleware, opts[:adapter])\n  end\nend\n```\n\nNow, you can create clients with different configurations:\n\n```elixir\nclient = MyApp.ServiceName.client(\n  base_url: \"https://api.service.com\",\n  bearer_token: \"token_value\",\n  adapter: Tesla.Adapter.Hackney\n  # Additional options...\n)\n{:ok, response} = MyApp.ServiceName.operation_name(client, %{key: \"value\"})\n```","ref":"0-client.html#multi-client-pattern","title":"Multi-Client Pattern - Client","type":"extras"},{"doc":"The choice between using a single-client (singleton) or multi-client pattern\ndepends on your specific needs:\n\n- **Library Authors**: It's generally advisable to avoid the singleton client\n  pattern. Hardcoding configurations can limit flexibility and hinder users in\n  multi-tenant environments. Providing the ability to create clients with custom\n  configurations makes your library more adaptable and user-friendly.\n\n- **Application Developers**: For simpler applications, a singleton client might\n  suffice initially. However, adopting the multi-client approach from the outset\n  can prevent future refactoring if your application grows or needs change.\n\nUnderstanding these patterns helps you design applications and libraries that\nare flexible and maintainable, aligning with best practices in software\ndevelopment.","ref":"0-client.html#comparing-single-client-and-multi-client-patterns","title":"Comparing Single-Client and Multi-Client Patterns - Client","type":"extras"},{"doc":"# Testing\n\nThere are two primary ways to mock requests in Tesla:\n\n- Using `Mox`\n- Using `Tesla.Mock` (deprecated)\n\nYou can also create a custom mock adapter if needed. For more information about\nadapters, refer to the [Adapter Guide](./3.adapter.md) to create your own.","ref":"1-testing.html","title":"Testing","type":"extras"},{"doc":"We recommend using `Mox` for mocking requests in tests because it\nis well-established in the Elixir community and provides robust features for\nconcurrent testing. While `Tesla.Mock` offers useful capabilities, it may be\nremoved in future releases. Consider using `Mox` to ensure long-term\ncompatibility.\nFor additional context, see [GitHub Issue #241](https://github.com/elixir-tesla/tesla/issues/241).","ref":"1-testing.html#should-i-use-mox-or-tesla-mock","title":"Should I Use Mox or Tesla.Mock? - Testing","type":"extras"},{"doc":"- [How-To Test Using Mox](../howtos/test-using-mox.md)","ref":"1-testing.html#references","title":"References - Testing","type":"extras"},{"doc":"# Middleware\n\nTL;DR: `adapter(middleware3(middleware2(middleware1(env, next, options))))`\n\nMiddleware in `Tesla` extends the request/response pipeline. Requests pass\nthrough a stack of middleware before reaching the adapter, allowing\nmodifications to both requests and responses.\n\nMiddleware can be a module implementing `Tesla.Middleware`\nbehaviour or a function matching `c:Tesla.Middleware.call/3`. There is no\ndistinction between request and response middleware; it's about when you\nexecute `Tesla.run/2`.\n\nThe middleware stack is processed by calling `Tesla.run/2` until it reaches\nthe adapter.","ref":"2-middleware.html","title":"Middleware","type":"extras"},{"doc":"Example of a custom middleware module:\n\n```elixir\ndefmodule Tesla.Middleware.MyCustomMiddleware do\n  @behaviour Tesla.Middleware\n\n  @impl Tesla.Middleware\n  def call(env, next, options) do\n    # Actions before calling the next middleware\n    # ...\n    Tesla.run(env, next)\n    # Actions after calling the next middleware\n    # ...\n  end\nend\n```\n\nA request logger middleware example:\n\n```elixir\ndefmodule MyApp.Tesla.Middleware.Logger do\n  require Logger\n\n  @behaviour Tesla.Middleware\n\n  def call(env, next, _) do\n    Logger.info(\"Request: #{inspect(env)}\")\n    case Tesla.run(env, next) do\n      {:ok, env} ->\n        Logger.info(\"Response: #{inspect(env)}\")\n        {:ok, env}\n\n      {:error, reason} ->\n        Logger.error(\"Error: #{inspect(reason)}\")\n        {:error, reason}\n    end\n  end\nend\n```","ref":"2-middleware.html#writing-middleware","title":"Writing Middleware - Middleware","type":"extras"},{"doc":"In a production application, you might combine built-in and custom middleware.\nHere's an example pipeline:\n\n```elixir\ndefmodule MyApp.ServiceName do\n  defp middleware do\n    base_url = \"...\"\n    token = \"...\"\n\n    [\n      # Preserve the original request, should be the first middleware in the\n      # pipeline to preserve the original request.\n      Tesla.Middleware.KeepRequest,\n\n      # Set the base URL\n      {Tesla.Middleware.BaseUrl, base_url},\n\n      # Add headers\n      {Tesla.Middleware.Headers, [{\"user-agent\", \"MyApp/1.0\"}]},\n      # Add authorization\n      {Tesla.Middleware.BearerAuth, [token: token]},\n      # Process the body (encoding, compression)\n      # Use string keys for untrusted input\n      Tesla.Middleware.JSON,\n      # Compress the request and response\n      # Tesla.Middleware.Compression,\n\n      # Optional OpenTelemetry middleware. Be careful adding `Tesla.Middleware.PathParams`\n      # before this middleware since you want to have low cardinality attribute\n      # values, therefore, you want the URL template.\n      # Tesla.Middleware.OpenTelemetry\n\n      # Keep the telemetry and logging as close as possible to the actual request\n      # being made.\n      # Log requests and responses\n      Tesla.Middleware.Logger,\n      # Telemetry of the Request\n      Tesla.Middleware.Telemetry,\n\n      # Replaces the Path Params, you may want keep it at the end when telemetry\n      # packages prefer to work with the URL template.\n      Tesla.Middleware.PathParams,\n    ]\n  end\nend\n```\n\nSee [built-in middlewares](https://github.com/elixir-tesla/tesla/tree/master/lib/tesla/middleware)\nfor more examples.","ref":"2-middleware.html#production-ready-middleware-pipeline-example","title":"Production-Ready Middleware Pipeline Example - Middleware","type":"extras"},{"doc":"# Adapter\n\nAn adapter in Tesla implements the `Tesla.Adapter` behaviour and handles the\nactual HTTP communication. It's the final step in the middleware chain,\nresponsible for sending requests and receiving responses.","ref":"3-adapter.html","title":"Adapter","type":"extras"},{"doc":"You can create a custom adapter by implementing the `Tesla.Adapter` behaviour.\nHere's an example:\n\n```elixir\ndefmodule Tesla.Adapter.Req do\n  @behaviour Tesla.Adapter\n\n  @impl Tesla.Adapter\n  def call(env, _opts) do\n    req = Req.new(\n      url: Tesla.build_url(env),\n      method: env.method,\n      headers: env.headers,\n      body: env.body\n    )\n\n    case Req.request(req) do\n      {:ok, %Req.Response{} = resp} ->\n        {:ok, %Tesla.Env{env | status: resp.status, headers: resp.headers, body: resp.body}}\n\n      {:error, reason} ->\n        {:error, reason}\n    end\n  end\nend\n```","ref":"3-adapter.html#writing-an-adapter","title":"Writing an Adapter - Adapter","type":"extras"},{"doc":"If you don't specify an adapter when creating a client with `Tesla.client/2`,\n`Tesla` uses the adapter configured in the `:tesla` application environment.\nBy default, Tesla uses `Tesla.Adapter.Httpc`, which relies on Erlang's built-in\n`httpc`.\n\n> #### :httpc as default Adapter {: .error}\n> The default `httpc` adapter is not recommended for production because it\n> doesn't validate SSL certificates and has other issues. Consider using `Mint`,\n> `Finch`, or `Hackney` adapters instead.","ref":"3-adapter.html#setting-the-adapter","title":"Setting the Adapter - Adapter","type":"extras"},{"doc":"You can pass options to adapters in several ways:\n\n- In the application configuration:\n\n  ```elixir\n  config :tesla, adapter: {Tesla.Adapter.Hackney, [recv_timeout: 30_000]}\n  ```\n\n- When creating a client:\n\n  ```elixir\n  defmodule MyService do\n    def client(...) do\n      middleware = [...]\n      adapter = {Tesla.Adapter.Hackney, [recv_timeout: 30_000]}\n      Tesla.client(middleware, adapter)\n    end\n  end\n  ```\n\n- Directly in request functions:\n\n  ```elixir\n  Tesla.get(client, \"/\", opts: [adapter: [recv_timeout: 30_000]])\n  ```","ref":"3-adapter.html#adapter-options","title":"Adapter Options - Adapter","type":"extras"},{"doc":"[People have complained about `:httpc` adapter in `Tesla` due to\nits security issues. The main problem is that `:httpc` does not validate SSL\ncertificates by default][0]. Which, we believe, is a serious security issue and\nshould be addressed by `:httpc` itself.\n\nAs much as we would like to fix it, we can't, because we are unsure if it would\nbreak existing code. We are not planning to fix it in `Tesla` due to backward\ncompatibility. We may reconsider this decision for a version 2.0.\n\n[0]: https://github.com/elixir-tesla/tesla/issues/293","ref":"3-adapter.html#about-httpc-adapter-and-security-issues","title":"About :httpc adapter and security issues - Adapter","type":"extras"},{"doc":"# Basic Cheat Sheet","ref":"general.html","title":"Basic Cheat Sheet","type":"extras"},{"doc":"","ref":"general.html#making-requests-101","title":"Making Requests 101 - Basic Cheat Sheet","type":"extras"},{"doc":"```elixir\nclient = Tesla.client([{Tesla.Middleware.BaseUrl, \"https://httpbin.org\"}])\nTesla.get(client, \"/path\")\n```","ref":"general.html#creating-a-client","title":"Creating a client - Basic Cheat Sheet","type":"extras"},{"doc":"```elixir\nTesla.get(\"https://httpbin.org/get\")\n\nTesla.head(\"https://httpbin.org/anything\")\nTesla.options(\"https://httpbin.org/anything\")\nTesla.trace(\"https://httpbin.org/anything\")\n\nTesla.post(\"https://httpbin.org/post\", \"body\")\nTesla.put(\"https://httpbin.org/put\", \"body\")\nTesla.patch(\"https://httpbin.org/patch\", \"body\")\nTesla.delete(\"https://httpbin.org/anything\")\n```","ref":"general.html#all-methods","title":"All Methods - Basic Cheat Sheet","type":"extras"},{"doc":"```elixir\n# GET /path?a=hi&b[]=1&b[]=2&b[]=3\nTesla.get(\"https://httpbin.org/anything\", query: [a: \"hi\", b: [1, 2, 3]])\n```","ref":"general.html#query-params","title":"Query Params - Basic Cheat Sheet","type":"extras"},{"doc":"```elixir\nTesla.get(\"https://httpbin.org/headers\", headers: [{\"x-api-key\", \"1\"}])\n```","ref":"general.html#request-headers","title":"Request Headers - Basic Cheat Sheet","type":"extras"},{"doc":"```elixir\nclient = Tesla.client([{Tesla.Middleware.Headers, [{\"user-agent\", \"Tesla\"}]}])\n```","ref":"general.html#client-default-headers","title":"Client Default Headers - Basic Cheat Sheet","type":"extras"},{"doc":"You can pass a `Tesla.Multipart` struct as the body:\n\n```elixir\nalias Tesla.Multipart\n\nmp =\n  Multipart.new()\n  |> Multipart.add_content_type_param(\"charset=utf-8\")\n  |> Multipart.add_field(\"field1\", \"foo\")\n  |> Multipart.add_field(\"field2\", \"bar\",\n    headers: [{\"content-id\", \"1\"}, {\"content-type\", \"text/plain\"}]\n  )\n  |> Multipart.add_file(\"test/tesla/multipart_test_file.sh\")\n  |> Multipart.add_file(\"test/tesla/multipart_test_file.sh\", name: \"foobar\")\n  |> Multipart.add_file_content(\"sample file content\", \"sample.txt\")\n\n{:ok, response} = Tesla.post(\"https://httpbin.org/post\", mp)\n```","ref":"general.html#multipart","title":"Multipart - Basic Cheat Sheet","type":"extras"},{"doc":"","ref":"general.html#streaming","title":"Streaming - Basic Cheat Sheet","type":"extras"},{"doc":"If adapter supports it, you can pass a [Stream](https://hexdocs.pm/elixir/main/Stream.html)\nas request body, e.g.:\n\n```elixir\ndefmodule ElasticSearch do\n  def index(records_stream) do\n    stream = Stream.map(records_stream, fn record -> %{index: [some, data]} end)\n    Tesla.post(client(), \"/_bulk\", stream)\n  end\n\n  defp client do\n    Tesla.client([\n      {Tesla.Middleware.BaseUrl, \"http://localhost:9200\"},\n      Tesla.Middleware.JSON\n    ], {Tesla.Adapter.Finch, name: MyFinch})\n  end\nend\n```","ref":"general.html#streaming-request-body","title":"Streaming Request Body - Basic Cheat Sheet","type":"extras"},{"doc":"If adapter supports it, you can pass a `response: :stream` option to return\nresponse body as a [Stream](https://hexdocs.pm/elixir/main/Stream.html)\n\n```elixir\ndefmodule OpenAI do\n  def client(token) do\n    middleware = [\n      {Tesla.Middleware.BaseUrl, \"https://api.openai.com/v1\"},\n      {Tesla.Middleware.BearerAuth, token: token},\n      {Tesla.Middleware.JSON, decode_content_types: [\"text/event-stream\"]},\n      {Tesla.Middleware.SSE, only: :data}\n    ]\n    Tesla.client(middleware, {Tesla.Adapter.Finch, name: MyFinch})\n  end\n\n  def completion(client, prompt) do\n    data = %{\n      model: \"gpt-3.5-turbo\",\n      messages: [%{role: \"user\", content: prompt}],\n      stream: true\n    }\n    Tesla.post(client, \"/chat/completions\", data, opts: [adapter: [response: :stream]])\n  end\nend\n\nclient = OpenAI.new(\" \")\n{:ok, env} = OpenAI.completion(client, \"What is the meaning of life?\")\nenv.body |> Stream.each(fn chunk -> IO.inspect(chunk) end)\n```","ref":"general.html#streaming-response-body","title":"Streaming Response Body - Basic Cheat Sheet","type":"extras"},{"doc":"","ref":"general.html#middleware","title":"Middleware - Basic Cheat Sheet","type":"extras"},{"doc":"```elixir\ndefmodule Tesla.Middleware.MyCustomMiddleware do\n  @moduledoc \"\"\"\n  Short description what it does\n\n  Longer description, including e.g. additional dependencies.","ref":"general.html#custom-middleware","title":"Custom middleware - Basic Cheat Sheet","type":"extras"},{"doc":"- `:list` - all possible options\n  - `:with` - their default values","ref":"general.html#options","title":"Options - Basic Cheat Sheet","type":"extras"},{"doc":"client = Tesla.client([{Tesla.Middleware.MyCustomMiddleware, with: value}])\n  \"\"\"\n\n  @behaviour Tesla.Middleware\n\n  @impl Tesla.Middleware\n  def call(env, next, options) do\n    with %Tesla.Env{} = env <- preprocess(env) do\n      env\n      |> Tesla.run(next)\n      |> postprocess()\n    end\n  end\n\n  defp preprocess(env) do\n    env\n  end\n\n  defp postprocess({:ok, env}) do\n    {:ok, env}\n  end\n\n  def postprocess({:error, reason}) do\n    {:error, reason}\n  end\nend\n```","ref":"general.html#examples","title":"Examples - Basic Cheat Sheet","type":"extras"},{"doc":"","ref":"general.html#adapter","title":"Adapter - Basic Cheat Sheet","type":"extras"},{"doc":"```elixir\ndefmodule Tesla.Adapter.MyCustomAdapter do\n  @behaviour Tesla.Adapter\n\n  @impl Tesla.Adapter\n  def run(env, opts) do\n    # do something\n  end\nend\n```","ref":"general.html#custom-adapter","title":"Custom adapter - Basic Cheat Sheet","type":"extras"},{"doc":"# Migrate from v0 to v1\n\nThis is a list of all breaking changes.\n\nVersion `1.0` has been released, try it today!\n\n```elixir\n  defp deps do\n    [\n      {:tesla, \"1.0.0\"}\n    ]\n  end\n```\n\nAny other breaking change not on this list is considered a bug - in you find one please create a new issue.","ref":"v0-to-v1.html","title":"Migrate from v0 to v1","type":"extras"},{"doc":"`get(..)`, `post(..)`, etc. now return `{:ok, Tesla.Env} | {:error, reason}` ([#177](https://github.com/elixir-tesla/tesla/issues/177))\n\nIn `0.x` all http functions returned either `Tesla.Env` or raised an error.\nIn `1.0` these functions return ok/error tuples. The old behaviour can be achieved with the new `! (bang)` functions: `get!(...)`, `post!(...)`, etc.\n\n```elixir\ncase MyApi.get(\"/\") do\n  {:ok, %Tesla.Env{status: 200}} -> # ok response\n  {:ok, %Tesla.Env{status: 500}} -> # server error\n  {:error, reason} -> # connection & other errors\nend\n```","ref":"v0-to-v1.html#returning-tuple-result-from-http-functions","title":"Returning Tuple Result from HTTP Functions - Migrate from v0 to v1","type":"extras"},{"doc":"Use full module name for middleware and adapters.\n\n```diff\n# middleware\n-  plug :json\n+  plug Tesla.Middleware.JSON\n\n# adapter\n-  adapter :hackney\n+  adapter Tesla.Adapter.Hackney\n\n# config\n-  config :tesla, adapter: :mock\n+  config :tesla, adapter: Tesla.Mock\n```","ref":"v0-to-v1.html#dropped-aliases-support-159","title":"Dropped aliases support (#159) - Migrate from v0 to v1","type":"extras"},{"doc":"Extract functionality into separate module.\n\n```diff\n defmodule MyClient do\n-  plug :some_local_fun\n-\n-  def some_local_fun(env, next) do\n     # implementation\n-  end\n end\n\n+defmodule ProperlyNamedMiddleware do\n+  @behaviour Tesla.Middleware\n+  def call(env, next, _opts) do\n     # implementation\n+  end\n+end\n\n defmodule MyClient do\n+  plug ProperlyNamedMiddleware\n end\n```","ref":"v0-to-v1.html#dropped-local-middleware-adapter-functions-171","title":"Dropped local middleware/adapter functions (#171) - Migrate from v0 to v1","type":"extras"},{"doc":"This is very unlikely, but... if you hacked around with custom functions as client (the first argument) you need to stop.\nSee `Tesla.client/2` instead.","ref":"v0-to-v1.html#dropped-client-as-function-176","title":"Dropped client as function (#176) - Migrate from v0 to v1","type":"extras"},{"doc":"In `0.x` `env.headers` are a `map(binary => binary)`.\n\nIn `1.x` `env.headers` are a `[{binary, binary}]`.\n\nThis change also applies to middleware headers.\n\n#### Setting a header\n\n```diff\n-  env\n-  |> Map.update!(&Map.put(&1.headers, \"name\", \"value\"))\n\n+  env\n+  |> Tesla.put_header(\"name\", \"value\")\n```\n\n#### Getting a header\n```diff\n-  env.headers[\"cookie\"]\n+  Tesla.get_header(env, \"cookie\") # => \"secret\"\n+  Tesla.get_headers(env, \"cookie\") # => [\"secret\", \"token\", \"and more\"]\n\n\n-  case env.headers do\n-    %{\"server\" => server} -> ...\n-    _ -> ...\n-  end\n+  case Tesla.get_header(env, \"server\") do\n+    nil -> ...\n+    server ->\n+  end\n```\n\nThere are five new functions to deal with headers:\n- `Tesla.get_header(env, name) :: binary | nil` - Get first header with given `name`\n- `Tesla.get_headers(env, name)` :: [binary] - Get all headers values with given `name`\n- `Tesla.put_header(env, name, value)` - Set header with given `name` and `value`. Existing header with the same name will be overwritten.\n- `Tesla.put_headers(env, list)` - Add headers to the end of `env.headers`. Does **not** make the headers unique.\n- `Tesla.delete_header(env, name)` - Delete all headers with given `name`","ref":"v0-to-v1.html#headers-are-now-a-list-160","title":"Headers are now a list (#160) - Migrate from v0 to v1","type":"extras"},{"doc":"Tesla `1.0` works only with Elixir 1.4 or newer","ref":"v0-to-v1.html#dropped-support-for-elixir-1-3-164","title":"Dropped support for Elixir 1.3 (#164) - Migrate from v0 to v1","type":"extras"},{"doc":"```diff\n- MyClient.get(\"/\", opts: [recv_timeout: 30_000])\n+ MyClient.get(\"/\", opts: [adapter: [recv_timeout: 30_000]])\n```","ref":"v0-to-v1.html#adapter-options-need-to-be-wrapped-in-adapter-key","title":"Adapter options need to be wrapped in :adapter key: - Migrate from v0 to v1","type":"extras"},{"doc":"Debugging request and response details has been merged into a single Logger middleware. See `Tesla.Middleware.Logger` documentation for more information.\n\n```diff\n  defmodule MyClient do\n    use Tesla\n\n    plug Tesla.Middleware.Logger\n-   plug Tesla.Middleware.DebugLogger\n  end\n```","ref":"v0-to-v1.html#debuglogger-merged-into-logger-150","title":"DebugLogger merged into Logger (#150) - Migrate from v0 to v1","type":"extras"},{"doc":"The `Tesla.Middleware.JSON` now requires [jason](https://github.com/michalmuskala/jason) by default. If you want to keep using poison you will have to set `:engine` option - see [documentation](https://hexdocs.pm/tesla/Tesla.Middleware.JSON.html#module-example-usage) for details.","ref":"v0-to-v1.html#jason-is-the-new-default-json-library-175","title":"Jason is the new default JSON library (#175) - Migrate from v0 to v1","type":"extras"},{"doc":"# Migrate away from v1 Macro\n\nWe encourage users to contribute to this guide to help others migrate away from\nthe `v1` macro syntax. Every case is different, so we can't provide a\none-size-fits-all solution, but we can provide a guide to help you migrate your\ncodebase.\nPlease share your learnings and suggestions in the [Migrating away from v1 Macro GitHub Discussion](https://github.com/elixir-tesla/tesla/discussions/732).\n\n1. Find all the modules that use `use Tesla`\n\n   ```elixir\n   defmodule MyApp.MyTeslaClient do\n     use Tesla # <- this line\n   end\n   ```\n\n2. Remove `use Tesla`\n\n   ```diff\n    - defmodule MyApp.MyTeslaClient do\n    -   use Tesla\n    - end\n    + defmodule MyApp.MyTeslaClient do\n    + end\n   ```\n\n3. Find all the `plug` macro calls:\n\n   ```elixir\n   defmodule MyApp.MyTeslaClient do\n     plug Tesla.Middleware.KeepRequest # <- this line\n     plug Tesla.Middleware.PathParams # <- this line\n     plug Tesla.Middleware.JSON # <- this line\n   end\n   ```\n\n4. Move all the `plug` macro calls to a function that returns the middleware.\n\n   ```diff\n   defmodule MyApp.MyTeslaClient do\n   - plug Tesla.Middleware.KeepRequest\n   - plug Tesla.Middleware.PathParams\n   - plug Tesla.Middleware.JSON\n   +\n   + def middleware do\n   +   [Tesla.Middleware.KeepRequest, Tesla.Middleware.PathParams, Tesla.Middleware.JSON]\n   + end\n   end\n   ```\n\n5. Find all the `adapter` macro calls:\n\n   ```elixir\n   defmodule MyApp.MyTeslaClient do\n     adapter Tesla.Adapter.Hackney # <- this line\n     adapter fn env -> # <- or this line\n     end\n   end\n   ```\n\n6. Create a `adapter/0` function that returns the adapter to use for that given\n   module, or however you prefer to configure the adapter used. Please refer to\n   the [Adapter Explanation](../explanations/3.adapter.md) documentation for more\n   information.\n\n   > #### Context Matters {: .warning}\n   >\n   > **This step is probably the most important one.** The context in which the\n   > adapter is used matters a lot. Please be careful with this step, and test\n   > your changes thoroughly.\n\n   ```diff\n   defmodule MyApp.MyTeslaClient do\n   - adapter Tesla.Adapter.Hackney\n   + defp adapter do\n   +   # if the value is `nil`, the default global Tesla adapter will be used\n   +   # which is the existing behavior.\n   +   :my_app\n   +   |> Application.get_env(__MODULE__, [])\n   +   |> Keyword.get(:adapter)\n   + end\n   end\n   ```\n\n7. Create a `client/0` function that returns a `Tesla.Client` struct with the\n   middleware and adapter. Please refer to the [Client Explanation](../explanations/0.client.md)\n   documentation for more information.\n\n   ```elixir\n   defmodule MyApp.MyTeslaClient do\n     def client do\n       Tesla.client(middleware(), adapter())\n     end\n\n     defp middleware do\n       [Tesla.Middleware.KeepRequest, Tesla.Middleware.PathParams, Tesla.Middleware.JSON]\n     end\n\n     defp adapter do\n       :my_app\n       |> Application.get_env(__MODULE__, [])\n       |> Keyword.get(:adapter)\n     end\n   end\n   ```\n\n8. Replace all the `Tesla.get/2`, `Tesla.post/2`, etc. to receive the client\n   as an argument.\n\n   ```diff\n   defmodule MyApp.MyTeslaClient do\n     def do_something do\n   -   get(\"/endpoint\")\n   +   Tesla.get!(client(), \"/endpoint\")\n     end\n   end\n   ```","ref":"v1-macro-migration.html","title":"Migrate away from v1 Macro","type":"extras"},{"doc":"# Test Using Mox\n\nTo mock HTTP requests in your tests using Mox with the Tesla HTTP client,\nfollow these steps:","ref":"test-using-mox.html","title":"Test Using Mox","type":"extras"},{"doc":"First, define a mock adapter that implements the Tesla.Adapter behaviour. This\nadapter will intercept HTTP requests during testing.\n\nCreate a file at `test/support/mocks.ex`:\n\n```elixir\n# test/support/mocks.ex\nMox.defmock(MyApp.MockAdapter, for: Tesla.Adapter)\n```","ref":"test-using-mox.html#1-define-a-mock-adapter","title":"1. Define a Mock Adapter - Test Using Mox","type":"extras"},{"doc":"In your `config/test.exs` file, configure Tesla to use the mock adapter you\njust defined:\n\n```elixir\n# config/test.exs\nconfig :tesla, adapter: MyApp.MockAdapter\n```\n\nIf you are not using the global adapter configuration, ensure that your Tesla\nclient modules are configured to use `MyApp.MockAdapter` during tests.","ref":"test-using-mox.html#2-configure-the-mock-adapter-for-tests","title":"2. Configure the Mock Adapter for Tests - Test Using Mox","type":"extras"},{"doc":"Create a test module, for example `test/demo_test.exs`, and set up `Mox` to\ndefine expectations and verify them:\n\n```elixir\ndefmodule MyApp.FeatureTest do\n  use ExUnit.Case, async: true\n\n  import Mox\n  import Tesla.Test\n\n  setup :set_mox_from_context\n  setup :verify_on_exit!\n\n  test \"example test\" do\n    #--------- Given - Stubs and Preconditions\n    # Expect a single HTTP request to be made and return a JSON response\n    expect_tesla_call(\n      times: 1,\n      returns: json(%Tesla.Env{status: 200}, %{id: 1})\n    )\n\n    #--------- When - Run the code under test\n    # Make the HTTP request using Tesla\n    # Mimic a use case where we create a user\n    assert :ok = create_user!(%{username: \"johndoe\"})\n\n    #--------- Then - Assert postconditions\n    # Verify that the HTTP request was made and matches the expected parameters\n    assert_received_tesla_call(env, [])\n    assert env.status == 200\n    assert env.method == :post\n    assert env.url == \"https://acme.com/users\"\n    # ...\n\n    # Or you can verify the entire `t:Tesla.Env.t/0` using something like this:\n    assert_tesla_env(env, %Tesla.Env{\n      method: :post,\n      url: \"https://acme.com/users\",\n      body: %{username: \"johndoe\"},\n      status: 200,\n    })\n\n    # Verify that the mailbox is empty, indicating no additional requests were\n    # made and all messages have been processed\n    assert_tesla_empty_mailbox()\n  end\n\n  defp create_user!(body) do\n    # ...\n    Tesla.post!(\"https://acme.com/users\", body)\n    # ...\n    :ok\n  end\nend\n```","ref":"test-using-mox.html#3-set-up-mocking-in-your-tests","title":"3. Set Up Mocking in Your Tests - Test Using Mox","type":"extras"},{"doc":"When you run your tests with `mix test`, all HTTP requests made by Tesla will\nbe intercepted by `MyApp.MockAdapter`, and responses will be provided based\non your `Mox` expectations.","ref":"test-using-mox.html#4-run-your-tests","title":"4. Run Your Tests - Test Using Mox","type":"extras"}],"producer":{"name":"ex_doc","version":"0.38.2"},"proglang":"elixir"}